/* libguestfs generated file
 * WARNING: THIS FILE IS GENERATED FROM:
 *   generator/generator_*.ml
 * ANY CHANGES YOU MAKE TO THIS FILE WILL BE LOST.
 *
 * Copyright (C) 2009-2010 Red Hat Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <inttypes.h>

#include "guestfs.h"
#include "guestfs-internal.h"
#include "guestfs-internal-actions.h"
#include "guestfs_protocol.h"

/* Check the return message from a call for validity. */
static int
check_reply_header (guestfs_h *g,
                    const struct guestfs_message_header *hdr,
                    unsigned int proc_nr, unsigned int serial)
{
  if (hdr->prog != GUESTFS_PROGRAM) {
    error (g, "wrong program (%d/%d)", hdr->prog, GUESTFS_PROGRAM);
    return -1;
  }
  if (hdr->vers != GUESTFS_PROTOCOL_VERSION) {
    error (g, "wrong protocol version (%d/%d)",
           hdr->vers, GUESTFS_PROTOCOL_VERSION);
    return -1;
  }
  if (hdr->direction != GUESTFS_DIRECTION_REPLY) {
    error (g, "unexpected message direction (%d/%d)",
           hdr->direction, GUESTFS_DIRECTION_REPLY);
    return -1;
  }
  if (hdr->proc != proc_nr) {
    error (g, "unexpected procedure number (%d/%d)", hdr->proc, proc_nr);
    return -1;
  }
  if (hdr->serial != serial) {
    error (g, "unexpected serial (%d/%d)", hdr->serial, serial);
    return -1;
  }

  return 0;
}

/* Check we are in the right state to run a high-level action. */
static int
check_state (guestfs_h *g, const char *caller)
{
  if (!guestfs__is_ready (g)) {
    if (guestfs__is_config (g) || guestfs__is_launching (g))
      error (g, "%s: call launch before using this function\n(in guestfish, don't forget to use the 'run' command)",
        caller);
    else
      error (g, "%s called from the wrong state, %d != READY",
        caller, guestfs__get_state (g));
    return -1;
  }
  return 0;
}

int guestfs_test0 (guestfs_h *g,
		const char *str,
		const char *optstr,
		char *const *strlist,
		int b,
		int integer,
		int64_t integer64,
		const char *filein,
		const char *fileout,
		const char *bufferin,
		size_t bufferin_size)
{
  if (str == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0", "str");
    return -1;
  }
  if (strlist == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0", "strlist");
    return -1;
  }
  if (filein == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0", "filein");
    return -1;
  }
  if (fileout == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0", "fileout");
    return -1;
  }
  if (bufferin == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0", "bufferin");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "test0");
    fprintf (stderr, " \"%s\"", str);
    if (optstr) fprintf (stderr, " \"%s\"", optstr);
    else fprintf (stderr, " null");
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; strlist[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (strlist[i], stderr);
    }
    fputc ('"', stderr);
    fputs (b ? " true" : " false", stderr);
    fprintf (stderr, " %d", integer);
    fprintf (stderr, " %" PRIi64, integer64);
    fprintf (stderr, " \"%s\"", filein);
    fprintf (stderr, " \"%s\"", fileout);
    fprintf (stderr, " \"%s\"", bufferin);
    fputc ('\n', stderr);
  }

  return guestfs__test0 (g, str, optstr, strlist, b, integer, integer64, filein, fileout, bufferin, bufferin_size);
}

int guestfs_test0rint (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rint", "val");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rint");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rint (g, val);
}

int guestfs_test0rinterr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rinterr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rinterr (g);
}

int64_t guestfs_test0rint64 (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rint64", "val");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rint64");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rint64 (g, val);
}

int64_t guestfs_test0rint64err (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rint64err");
    fputc ('\n', stderr);
  }

  return guestfs__test0rint64err (g);
}

int guestfs_test0rbool (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rbool", "val");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rbool");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rbool (g, val);
}

int guestfs_test0rboolerr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rboolerr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rboolerr (g);
}

const char *guestfs_test0rconststring (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rconststring", "val");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rconststring");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rconststring (g, val);
}

const char *guestfs_test0rconststringerr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rconststringerr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rconststringerr (g);
}

const char *guestfs_test0rconstoptstring (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rconstoptstring", "val");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rconstoptstring");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rconstoptstring (g, val);
}

const char *guestfs_test0rconstoptstringerr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rconstoptstringerr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rconstoptstringerr (g);
}

char *guestfs_test0rstring (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rstring", "val");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rstring");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rstring (g, val);
}

char *guestfs_test0rstringerr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rstringerr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rstringerr (g);
}

char **guestfs_test0rstringlist (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rstringlist", "val");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rstringlist");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rstringlist (g, val);
}

char **guestfs_test0rstringlisterr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rstringlisterr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rstringlisterr (g);
}

struct guestfs_lvm_pv *guestfs_test0rstruct (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rstruct", "val");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rstruct");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rstruct (g, val);
}

struct guestfs_lvm_pv *guestfs_test0rstructerr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rstructerr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rstructerr (g);
}

struct guestfs_lvm_pv_list *guestfs_test0rstructlist (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rstructlist", "val");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rstructlist");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rstructlist (g, val);
}

struct guestfs_lvm_pv_list *guestfs_test0rstructlisterr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rstructlisterr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rstructlisterr (g);
}

char **guestfs_test0rhashtable (guestfs_h *g,
		const char *val)
{
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "test0rhashtable", "val");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rhashtable");
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  return guestfs__test0rhashtable (g, val);
}

char **guestfs_test0rhashtableerr (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "test0rhashtableerr");
    fputc ('\n', stderr);
  }

  return guestfs__test0rhashtableerr (g);
}

int guestfs_launch (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "launch");
    fputc ('\n', stderr);
  }

  return guestfs__launch (g);
}

int guestfs_wait_ready (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "wait_ready");
    fputc ('\n', stderr);
  }

  return guestfs__wait_ready (g);
}

int guestfs_kill_subprocess (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "kill_subprocess");
    fputc ('\n', stderr);
  }

  return guestfs__kill_subprocess (g);
}

int guestfs_add_drive (guestfs_h *g,
		const char *filename)
{
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "add_drive", "filename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "add_drive");
    fprintf (stderr, " \"%s\"", filename);
    fputc ('\n', stderr);
  }

  return guestfs__add_drive (g, filename);
}

int guestfs_add_cdrom (guestfs_h *g,
		const char *filename)
{
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "add_cdrom", "filename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "add_cdrom");
    fprintf (stderr, " \"%s\"", filename);
    fputc ('\n', stderr);
  }

  return guestfs__add_cdrom (g, filename);
}

int guestfs_add_drive_ro (guestfs_h *g,
		const char *filename)
{
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "add_drive_ro", "filename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "add_drive_ro");
    fprintf (stderr, " \"%s\"", filename);
    fputc ('\n', stderr);
  }

  return guestfs__add_drive_ro (g, filename);
}

int guestfs_config (guestfs_h *g,
		const char *qemuparam,
		const char *qemuvalue)
{
  if (qemuparam == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "config", "qemuparam");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "config");
    fprintf (stderr, " \"%s\"", qemuparam);
    if (qemuvalue) fprintf (stderr, " \"%s\"", qemuvalue);
    else fprintf (stderr, " null");
    fputc ('\n', stderr);
  }

  return guestfs__config (g, qemuparam, qemuvalue);
}

int guestfs_set_qemu (guestfs_h *g,
		const char *qemu)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_qemu");
    if (qemu) fprintf (stderr, " \"%s\"", qemu);
    else fprintf (stderr, " null");
    fputc ('\n', stderr);
  }

  return guestfs__set_qemu (g, qemu);
}

const char *guestfs_get_qemu (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_qemu");
    fputc ('\n', stderr);
  }

  return guestfs__get_qemu (g);
}

int guestfs_set_path (guestfs_h *g,
		const char *searchpath)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_path");
    if (searchpath) fprintf (stderr, " \"%s\"", searchpath);
    else fprintf (stderr, " null");
    fputc ('\n', stderr);
  }

  return guestfs__set_path (g, searchpath);
}

const char *guestfs_get_path (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_path");
    fputc ('\n', stderr);
  }

  return guestfs__get_path (g);
}

int guestfs_set_append (guestfs_h *g,
		const char *append)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_append");
    if (append) fprintf (stderr, " \"%s\"", append);
    else fprintf (stderr, " null");
    fputc ('\n', stderr);
  }

  return guestfs__set_append (g, append);
}

const char *guestfs_get_append (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_append");
    fputc ('\n', stderr);
  }

  return guestfs__get_append (g);
}

int guestfs_set_autosync (guestfs_h *g,
		int autosync)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_autosync");
    fputs (autosync ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  return guestfs__set_autosync (g, autosync);
}

int guestfs_get_autosync (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_autosync");
    fputc ('\n', stderr);
  }

  return guestfs__get_autosync (g);
}

int guestfs_set_verbose (guestfs_h *g,
		int verbose)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_verbose");
    fputs (verbose ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  return guestfs__set_verbose (g, verbose);
}

int guestfs_get_verbose (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_verbose");
    fputc ('\n', stderr);
  }

  return guestfs__get_verbose (g);
}

int guestfs_is_ready (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_ready");
    fputc ('\n', stderr);
  }

  return guestfs__is_ready (g);
}

int guestfs_is_config (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_config");
    fputc ('\n', stderr);
  }

  return guestfs__is_config (g);
}

int guestfs_is_launching (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_launching");
    fputc ('\n', stderr);
  }

  return guestfs__is_launching (g);
}

int guestfs_is_busy (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_busy");
    fputc ('\n', stderr);
  }

  return guestfs__is_busy (g);
}

int guestfs_get_state (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_state");
    fputc ('\n', stderr);
  }

  return guestfs__get_state (g);
}

int guestfs_set_memsize (guestfs_h *g,
		int memsize)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_memsize");
    fprintf (stderr, " %d", memsize);
    fputc ('\n', stderr);
  }

  return guestfs__set_memsize (g, memsize);
}

int guestfs_get_memsize (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_memsize");
    fputc ('\n', stderr);
  }

  return guestfs__get_memsize (g);
}

int guestfs_get_pid (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_pid");
    fputc ('\n', stderr);
  }

  return guestfs__get_pid (g);
}

struct guestfs_version *guestfs_version (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "version");
    fputc ('\n', stderr);
  }

  return guestfs__version (g);
}

int guestfs_set_selinux (guestfs_h *g,
		int selinux)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_selinux");
    fputs (selinux ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  return guestfs__set_selinux (g, selinux);
}

int guestfs_get_selinux (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_selinux");
    fputc ('\n', stderr);
  }

  return guestfs__get_selinux (g);
}

int guestfs_set_trace (guestfs_h *g,
		int trace)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_trace");
    fputs (trace ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  return guestfs__set_trace (g, trace);
}

int guestfs_get_trace (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_trace");
    fputc ('\n', stderr);
  }

  return guestfs__get_trace (g);
}

int guestfs_set_direct (guestfs_h *g,
		int direct)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_direct");
    fputs (direct ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  return guestfs__set_direct (g, direct);
}

int guestfs_get_direct (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_direct");
    fputc ('\n', stderr);
  }

  return guestfs__get_direct (g);
}

int guestfs_set_recovery_proc (guestfs_h *g,
		int recoveryproc)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_recovery_proc");
    fputs (recoveryproc ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  return guestfs__set_recovery_proc (g, recoveryproc);
}

int guestfs_get_recovery_proc (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_recovery_proc");
    fputc ('\n', stderr);
  }

  return guestfs__get_recovery_proc (g);
}

int guestfs_add_drive_with_if (guestfs_h *g,
		const char *filename,
		const char *iface)
{
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "add_drive_with_if", "filename");
    return -1;
  }
  if (iface == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "add_drive_with_if", "iface");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "add_drive_with_if");
    fprintf (stderr, " \"%s\"", filename);
    fprintf (stderr, " \"%s\"", iface);
    fputc ('\n', stderr);
  }

  return guestfs__add_drive_with_if (g, filename, iface);
}

int guestfs_add_drive_ro_with_if (guestfs_h *g,
		const char *filename,
		const char *iface)
{
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "add_drive_ro_with_if", "filename");
    return -1;
  }
  if (iface == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "add_drive_ro_with_if", "iface");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "add_drive_ro_with_if");
    fprintf (stderr, " \"%s\"", filename);
    fprintf (stderr, " \"%s\"", iface);
    fputc ('\n', stderr);
  }

  return guestfs__add_drive_ro_with_if (g, filename, iface);
}

char *guestfs_file_architecture (guestfs_h *g,
		const char *filename)
{
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "file_architecture", "filename");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "file_architecture");
    fprintf (stderr, " \"%s\"", filename);
    fputc ('\n', stderr);
  }

  return guestfs__file_architecture (g, filename);
}

char **guestfs_inspect_os (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_os");
    fputc ('\n', stderr);
  }

  return guestfs__inspect_os (g);
}

char *guestfs_inspect_get_type (guestfs_h *g,
		const char *root)
{
  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inspect_get_type", "root");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_get_type");
    fprintf (stderr, " \"%s\"", root);
    fputc ('\n', stderr);
  }

  return guestfs__inspect_get_type (g, root);
}

char *guestfs_inspect_get_arch (guestfs_h *g,
		const char *root)
{
  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inspect_get_arch", "root");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_get_arch");
    fprintf (stderr, " \"%s\"", root);
    fputc ('\n', stderr);
  }

  return guestfs__inspect_get_arch (g, root);
}

char *guestfs_inspect_get_distro (guestfs_h *g,
		const char *root)
{
  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inspect_get_distro", "root");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_get_distro");
    fprintf (stderr, " \"%s\"", root);
    fputc ('\n', stderr);
  }

  return guestfs__inspect_get_distro (g, root);
}

int guestfs_inspect_get_major_version (guestfs_h *g,
		const char *root)
{
  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inspect_get_major_version", "root");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_get_major_version");
    fprintf (stderr, " \"%s\"", root);
    fputc ('\n', stderr);
  }

  return guestfs__inspect_get_major_version (g, root);
}

int guestfs_inspect_get_minor_version (guestfs_h *g,
		const char *root)
{
  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inspect_get_minor_version", "root");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_get_minor_version");
    fprintf (stderr, " \"%s\"", root);
    fputc ('\n', stderr);
  }

  return guestfs__inspect_get_minor_version (g, root);
}

char *guestfs_inspect_get_product_name (guestfs_h *g,
		const char *root)
{
  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inspect_get_product_name", "root");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_get_product_name");
    fprintf (stderr, " \"%s\"", root);
    fputc ('\n', stderr);
  }

  return guestfs__inspect_get_product_name (g, root);
}

char **guestfs_inspect_get_mountpoints (guestfs_h *g,
		const char *root)
{
  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inspect_get_mountpoints", "root");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_get_mountpoints");
    fprintf (stderr, " \"%s\"", root);
    fputc ('\n', stderr);
  }

  return guestfs__inspect_get_mountpoints (g, root);
}

char **guestfs_inspect_get_filesystems (guestfs_h *g,
		const char *root)
{
  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inspect_get_filesystems", "root");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inspect_get_filesystems");
    fprintf (stderr, " \"%s\"", root);
    fputc ('\n', stderr);
  }

  return guestfs__inspect_get_filesystems (g, root);
}

int guestfs_set_network (guestfs_h *g,
		int network)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_network");
    fputs (network ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  return guestfs__set_network (g, network);
}

int guestfs_get_network (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_network");
    fputc ('\n', stderr);
  }

  return guestfs__get_network (g);
}

char **guestfs_list_filesystems (guestfs_h *g)
{
  if (guestfs__get_trace (g)) {
    fprintf (stderr, "list_filesystems");
    fputc ('\n', stderr);
  }

  return guestfs__list_filesystems (g);
}

int guestfs_mount (guestfs_h *g,
		const char *device,
		const char *mountpoint)
{
  struct guestfs_mount_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount", "device");
    return -1;
  }
  if (mountpoint == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount", "mountpoint");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mount");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", mountpoint);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mount") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.mountpoint = (char *) mountpoint;
  serial = guestfs___send (g, GUESTFS_PROC_MOUNT,
        (xdrproc_t) xdr_guestfs_mount_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mount", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MOUNT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mount", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_sync (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "sync");
    fputc ('\n', stderr);
  }

  if (check_state (g, "sync") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_SYNC, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "sync", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SYNC, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sync", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_touch (guestfs_h *g,
		const char *path)
{
  struct guestfs_touch_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "touch", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "touch");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "touch") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_TOUCH,
        (xdrproc_t) xdr_guestfs_touch_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "touch", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TOUCH, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "touch", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_cat (guestfs_h *g,
		const char *path)
{
  struct guestfs_cat_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_cat_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "cat", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "cat");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "cat") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_CAT,
        (xdrproc_t) xdr_guestfs_cat_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "cat", &hdr, &err,
        (xdrproc_t) xdr_guestfs_cat_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CAT, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "cat", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.content; /* caller will free */
}

char *guestfs_ll (guestfs_h *g,
		const char *directory)
{
  struct guestfs_ll_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_ll_ret ret;
  int serial;
  int r;

  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ll", "directory");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ll");
    fprintf (stderr, " \"%s\"", directory);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ll") == -1) return NULL;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_LL,
        (xdrproc_t) xdr_guestfs_ll_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "ll", &hdr, &err,
        (xdrproc_t) xdr_guestfs_ll_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LL, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ll", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.listing; /* caller will free */
}

char **guestfs_ls (guestfs_h *g,
		const char *directory)
{
  struct guestfs_ls_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_ls_ret ret;
  int serial;
  int r;

  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ls", "directory");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ls");
    fprintf (stderr, " \"%s\"", directory);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ls") == -1) return NULL;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_LS,
        (xdrproc_t) xdr_guestfs_ls_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "ls", &hdr, &err,
        (xdrproc_t) xdr_guestfs_ls_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ls", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.listing.listing_val =
    safe_realloc (g, ret.listing.listing_val,
                  sizeof (char *) * (ret.listing.listing_len + 1));
  ret.listing.listing_val[ret.listing.listing_len] = NULL;
  return ret.listing.listing_val;
}

char **guestfs_list_devices (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_list_devices_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "list_devices");
    fputc ('\n', stderr);
  }

  if (check_state (g, "list_devices") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_LIST_DEVICES, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "list_devices", &hdr, &err,
        (xdrproc_t) xdr_guestfs_list_devices_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LIST_DEVICES, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "list_devices", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.devices.devices_val =
    safe_realloc (g, ret.devices.devices_val,
                  sizeof (char *) * (ret.devices.devices_len + 1));
  ret.devices.devices_val[ret.devices.devices_len] = NULL;
  return ret.devices.devices_val;
}

char **guestfs_list_partitions (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_list_partitions_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "list_partitions");
    fputc ('\n', stderr);
  }

  if (check_state (g, "list_partitions") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_LIST_PARTITIONS, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "list_partitions", &hdr, &err,
        (xdrproc_t) xdr_guestfs_list_partitions_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LIST_PARTITIONS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "list_partitions", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.partitions.partitions_val =
    safe_realloc (g, ret.partitions.partitions_val,
                  sizeof (char *) * (ret.partitions.partitions_len + 1));
  ret.partitions.partitions_val[ret.partitions.partitions_len] = NULL;
  return ret.partitions.partitions_val;
}

char **guestfs_pvs (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_pvs_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pvs");
    fputc ('\n', stderr);
  }

  if (check_state (g, "pvs") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_PVS, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "pvs", &hdr, &err,
        (xdrproc_t) xdr_guestfs_pvs_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PVS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pvs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.physvols.physvols_val =
    safe_realloc (g, ret.physvols.physvols_val,
                  sizeof (char *) * (ret.physvols.physvols_len + 1));
  ret.physvols.physvols_val[ret.physvols.physvols_len] = NULL;
  return ret.physvols.physvols_val;
}

char **guestfs_vgs (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_vgs_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vgs");
    fputc ('\n', stderr);
  }

  if (check_state (g, "vgs") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_VGS, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "vgs", &hdr, &err,
        (xdrproc_t) xdr_guestfs_vgs_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vgs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.volgroups.volgroups_val =
    safe_realloc (g, ret.volgroups.volgroups_val,
                  sizeof (char *) * (ret.volgroups.volgroups_len + 1));
  ret.volgroups.volgroups_val[ret.volgroups.volgroups_len] = NULL;
  return ret.volgroups.volgroups_val;
}

char **guestfs_lvs (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_lvs_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvs");
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvs") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_LVS, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "lvs", &hdr, &err,
        (xdrproc_t) xdr_guestfs_lvs_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.logvols.logvols_val =
    safe_realloc (g, ret.logvols.logvols_val,
                  sizeof (char *) * (ret.logvols.logvols_len + 1));
  ret.logvols.logvols_val[ret.logvols.logvols_len] = NULL;
  return ret.logvols.logvols_val;
}

struct guestfs_lvm_pv_list *guestfs_pvs_full (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_pvs_full_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pvs_full");
    fputc ('\n', stderr);
  }

  if (check_state (g, "pvs_full") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_PVS_FULL, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "pvs_full", &hdr, &err,
        (xdrproc_t) xdr_guestfs_pvs_full_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PVS_FULL, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pvs_full", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.physvols, sizeof (ret.physvols));
}

struct guestfs_lvm_vg_list *guestfs_vgs_full (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_vgs_full_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vgs_full");
    fputc ('\n', stderr);
  }

  if (check_state (g, "vgs_full") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_VGS_FULL, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "vgs_full", &hdr, &err,
        (xdrproc_t) xdr_guestfs_vgs_full_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGS_FULL, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vgs_full", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.volgroups, sizeof (ret.volgroups));
}

struct guestfs_lvm_lv_list *guestfs_lvs_full (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_lvs_full_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvs_full");
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvs_full") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_LVS_FULL, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "lvs_full", &hdr, &err,
        (xdrproc_t) xdr_guestfs_lvs_full_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVS_FULL, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvs_full", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.logvols, sizeof (ret.logvols));
}

char **guestfs_read_lines (guestfs_h *g,
		const char *path)
{
  struct guestfs_read_lines_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_read_lines_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "read_lines", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "read_lines");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "read_lines") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_READ_LINES,
        (xdrproc_t) xdr_guestfs_read_lines_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "read_lines", &hdr, &err,
        (xdrproc_t) xdr_guestfs_read_lines_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_READ_LINES, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "read_lines", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

int guestfs_aug_init (guestfs_h *g,
		const char *root,
		int flags)
{
  struct guestfs_aug_init_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_init", "root");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_init");
    fprintf (stderr, " \"%s\"", root);
    fprintf (stderr, " %d", flags);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_init") == -1) return -1;
  guestfs___set_busy (g);

  args.root = (char *) root;
  args.flags = flags;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_INIT,
        (xdrproc_t) xdr_guestfs_aug_init_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "aug_init", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_INIT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_init", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_aug_close (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_close");
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_close") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_AUG_CLOSE, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "aug_close", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_CLOSE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_close", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_aug_defvar (guestfs_h *g,
		const char *name,
		const char *expr)
{
  struct guestfs_aug_defvar_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_aug_defvar_ret ret;
  int serial;
  int r;

  if (name == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_defvar", "name");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_defvar");
    fprintf (stderr, " \"%s\"", name);
    if (expr) fprintf (stderr, " \"%s\"", expr);
    else fprintf (stderr, " null");
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_defvar") == -1) return -1;
  guestfs___set_busy (g);

  args.name = (char *) name;
  args.expr = expr ? (char **) &expr : NULL;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_DEFVAR,
        (xdrproc_t) xdr_guestfs_aug_defvar_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "aug_defvar", &hdr, &err,
        (xdrproc_t) xdr_guestfs_aug_defvar_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_DEFVAR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_defvar", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.nrnodes;
}

struct guestfs_int_bool *guestfs_aug_defnode (guestfs_h *g,
		const char *name,
		const char *expr,
		const char *val)
{
  struct guestfs_aug_defnode_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_aug_defnode_ret ret;
  int serial;
  int r;

  if (name == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_defnode", "name");
    return NULL;
  }
  if (expr == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_defnode", "expr");
    return NULL;
  }
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_defnode", "val");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_defnode");
    fprintf (stderr, " \"%s\"", name);
    fprintf (stderr, " \"%s\"", expr);
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_defnode") == -1) return NULL;
  guestfs___set_busy (g);

  args.name = (char *) name;
  args.expr = (char *) expr;
  args.val = (char *) val;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_DEFNODE,
        (xdrproc_t) xdr_guestfs_aug_defnode_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "aug_defnode", &hdr, &err,
        (xdrproc_t) xdr_guestfs_aug_defnode_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_DEFNODE, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_defnode", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.nrnodescreated, sizeof (ret.nrnodescreated));
}

char *guestfs_aug_get (guestfs_h *g,
		const char *augpath)
{
  struct guestfs_aug_get_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_aug_get_ret ret;
  int serial;
  int r;

  if (augpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_get", "augpath");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_get");
    fprintf (stderr, " \"%s\"", augpath);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_get") == -1) return NULL;
  guestfs___set_busy (g);

  args.augpath = (char *) augpath;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_GET,
        (xdrproc_t) xdr_guestfs_aug_get_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "aug_get", &hdr, &err,
        (xdrproc_t) xdr_guestfs_aug_get_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_GET, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_get", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.val; /* caller will free */
}

int guestfs_aug_set (guestfs_h *g,
		const char *augpath,
		const char *val)
{
  struct guestfs_aug_set_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (augpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_set", "augpath");
    return -1;
  }
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_set", "val");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_set");
    fprintf (stderr, " \"%s\"", augpath);
    fprintf (stderr, " \"%s\"", val);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_set") == -1) return -1;
  guestfs___set_busy (g);

  args.augpath = (char *) augpath;
  args.val = (char *) val;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_SET,
        (xdrproc_t) xdr_guestfs_aug_set_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "aug_set", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_SET, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_set", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_aug_insert (guestfs_h *g,
		const char *augpath,
		const char *label,
		int before)
{
  struct guestfs_aug_insert_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (augpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_insert", "augpath");
    return -1;
  }
  if (label == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_insert", "label");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_insert");
    fprintf (stderr, " \"%s\"", augpath);
    fprintf (stderr, " \"%s\"", label);
    fputs (before ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_insert") == -1) return -1;
  guestfs___set_busy (g);

  args.augpath = (char *) augpath;
  args.label = (char *) label;
  args.before = before;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_INSERT,
        (xdrproc_t) xdr_guestfs_aug_insert_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "aug_insert", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_INSERT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_insert", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_aug_rm (guestfs_h *g,
		const char *augpath)
{
  struct guestfs_aug_rm_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_aug_rm_ret ret;
  int serial;
  int r;

  if (augpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_rm", "augpath");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_rm");
    fprintf (stderr, " \"%s\"", augpath);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_rm") == -1) return -1;
  guestfs___set_busy (g);

  args.augpath = (char *) augpath;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_RM,
        (xdrproc_t) xdr_guestfs_aug_rm_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "aug_rm", &hdr, &err,
        (xdrproc_t) xdr_guestfs_aug_rm_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_RM, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_rm", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.nrnodes;
}

int guestfs_aug_mv (guestfs_h *g,
		const char *src,
		const char *dest)
{
  struct guestfs_aug_mv_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (src == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_mv", "src");
    return -1;
  }
  if (dest == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_mv", "dest");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_mv");
    fprintf (stderr, " \"%s\"", src);
    fprintf (stderr, " \"%s\"", dest);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_mv") == -1) return -1;
  guestfs___set_busy (g);

  args.src = (char *) src;
  args.dest = (char *) dest;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_MV,
        (xdrproc_t) xdr_guestfs_aug_mv_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "aug_mv", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_MV, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_mv", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char **guestfs_aug_match (guestfs_h *g,
		const char *augpath)
{
  struct guestfs_aug_match_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_aug_match_ret ret;
  int serial;
  int r;

  if (augpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_match", "augpath");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_match");
    fprintf (stderr, " \"%s\"", augpath);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_match") == -1) return NULL;
  guestfs___set_busy (g);

  args.augpath = (char *) augpath;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_MATCH,
        (xdrproc_t) xdr_guestfs_aug_match_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "aug_match", &hdr, &err,
        (xdrproc_t) xdr_guestfs_aug_match_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_MATCH, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_match", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.matches.matches_val =
    safe_realloc (g, ret.matches.matches_val,
                  sizeof (char *) * (ret.matches.matches_len + 1));
  ret.matches.matches_val[ret.matches.matches_len] = NULL;
  return ret.matches.matches_val;
}

int guestfs_aug_save (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_save");
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_save") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_AUG_SAVE, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "aug_save", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_SAVE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_save", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_aug_load (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_load");
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_load") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_AUG_LOAD, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "aug_load", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_LOAD, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_load", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char **guestfs_aug_ls (guestfs_h *g,
		const char *augpath)
{
  struct guestfs_aug_ls_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_aug_ls_ret ret;
  int serial;
  int r;

  if (augpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_ls", "augpath");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_ls");
    fprintf (stderr, " \"%s\"", augpath);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_ls") == -1) return NULL;
  guestfs___set_busy (g);

  args.augpath = (char *) augpath;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_LS,
        (xdrproc_t) xdr_guestfs_aug_ls_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "aug_ls", &hdr, &err,
        (xdrproc_t) xdr_guestfs_aug_ls_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_LS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_ls", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.matches.matches_val =
    safe_realloc (g, ret.matches.matches_val,
                  sizeof (char *) * (ret.matches.matches_len + 1));
  ret.matches.matches_val[ret.matches.matches_len] = NULL;
  return ret.matches.matches_val;
}

int guestfs_rm (guestfs_h *g,
		const char *path)
{
  struct guestfs_rm_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "rm", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "rm");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "rm") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_RM,
        (xdrproc_t) xdr_guestfs_rm_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "rm", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_RM, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "rm", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_rmdir (guestfs_h *g,
		const char *path)
{
  struct guestfs_rmdir_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "rmdir", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "rmdir");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "rmdir") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_RMDIR,
        (xdrproc_t) xdr_guestfs_rmdir_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "rmdir", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_RMDIR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "rmdir", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_rm_rf (guestfs_h *g,
		const char *path)
{
  struct guestfs_rm_rf_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "rm_rf", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "rm_rf");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "rm_rf") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_RM_RF,
        (xdrproc_t) xdr_guestfs_rm_rf_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "rm_rf", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_RM_RF, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "rm_rf", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkdir (guestfs_h *g,
		const char *path)
{
  struct guestfs_mkdir_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkdir", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkdir");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkdir") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_MKDIR,
        (xdrproc_t) xdr_guestfs_mkdir_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkdir", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKDIR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkdir", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkdir_p (guestfs_h *g,
		const char *path)
{
  struct guestfs_mkdir_p_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkdir_p", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkdir_p");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkdir_p") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_MKDIR_P,
        (xdrproc_t) xdr_guestfs_mkdir_p_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkdir_p", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKDIR_P, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkdir_p", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_chmod (guestfs_h *g,
		int mode,
		const char *path)
{
  struct guestfs_chmod_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "chmod", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "chmod");
    fprintf (stderr, " %d", mode);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "chmod") == -1) return -1;
  guestfs___set_busy (g);

  args.mode = mode;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_CHMOD,
        (xdrproc_t) xdr_guestfs_chmod_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "chmod", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CHMOD, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "chmod", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_chown (guestfs_h *g,
		int owner,
		int group,
		const char *path)
{
  struct guestfs_chown_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "chown", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "chown");
    fprintf (stderr, " %d", owner);
    fprintf (stderr, " %d", group);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "chown") == -1) return -1;
  guestfs___set_busy (g);

  args.owner = owner;
  args.group = group;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_CHOWN,
        (xdrproc_t) xdr_guestfs_chown_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "chown", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CHOWN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "chown", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_exists (guestfs_h *g,
		const char *path)
{
  struct guestfs_exists_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_exists_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "exists", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "exists");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "exists") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_EXISTS,
        (xdrproc_t) xdr_guestfs_exists_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "exists", &hdr, &err,
        (xdrproc_t) xdr_guestfs_exists_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_EXISTS, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "exists", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.existsflag;
}

int guestfs_is_file (guestfs_h *g,
		const char *path)
{
  struct guestfs_is_file_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_is_file_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "is_file", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_file");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "is_file") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_IS_FILE,
        (xdrproc_t) xdr_guestfs_is_file_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "is_file", &hdr, &err,
        (xdrproc_t) xdr_guestfs_is_file_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_IS_FILE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "is_file", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.fileflag;
}

int guestfs_is_dir (guestfs_h *g,
		const char *path)
{
  struct guestfs_is_dir_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_is_dir_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "is_dir", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_dir");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "is_dir") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_IS_DIR,
        (xdrproc_t) xdr_guestfs_is_dir_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "is_dir", &hdr, &err,
        (xdrproc_t) xdr_guestfs_is_dir_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_IS_DIR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "is_dir", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.dirflag;
}

int guestfs_pvcreate (guestfs_h *g,
		const char *device)
{
  struct guestfs_pvcreate_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pvcreate", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pvcreate");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pvcreate") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_PVCREATE,
        (xdrproc_t) xdr_guestfs_pvcreate_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "pvcreate", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PVCREATE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pvcreate", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_vgcreate (guestfs_h *g,
		const char *volgroup,
		char *const *physvols)
{
  struct guestfs_vgcreate_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (volgroup == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vgcreate", "volgroup");
    return -1;
  }
  if (physvols == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vgcreate", "physvols");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "vgcreate");
    fprintf (stderr, " \"%s\"", volgroup);
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; physvols[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (physvols[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vgcreate") == -1) return -1;
  guestfs___set_busy (g);

  args.volgroup = (char *) volgroup;
  args.physvols.physvols_val = (char **) physvols;
  for (args.physvols.physvols_len = 0; physvols[args.physvols.physvols_len]; args.physvols.physvols_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_VGCREATE,
        (xdrproc_t) xdr_guestfs_vgcreate_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "vgcreate", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGCREATE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vgcreate", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_lvcreate (guestfs_h *g,
		const char *logvol,
		const char *volgroup,
		int mbytes)
{
  struct guestfs_lvcreate_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (logvol == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvcreate", "logvol");
    return -1;
  }
  if (volgroup == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvcreate", "volgroup");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvcreate");
    fprintf (stderr, " \"%s\"", logvol);
    fprintf (stderr, " \"%s\"", volgroup);
    fprintf (stderr, " %d", mbytes);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvcreate") == -1) return -1;
  guestfs___set_busy (g);

  args.logvol = (char *) logvol;
  args.volgroup = (char *) volgroup;
  args.mbytes = mbytes;
  serial = guestfs___send (g, GUESTFS_PROC_LVCREATE,
        (xdrproc_t) xdr_guestfs_lvcreate_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lvcreate", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVCREATE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvcreate", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkfs (guestfs_h *g,
		const char *fstype,
		const char *device)
{
  struct guestfs_mkfs_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (fstype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkfs", "fstype");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkfs", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkfs");
    fprintf (stderr, " \"%s\"", fstype);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkfs") == -1) return -1;
  guestfs___set_busy (g);

  args.fstype = (char *) fstype;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_MKFS,
        (xdrproc_t) xdr_guestfs_mkfs_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkfs", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKFS, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkfs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_sfdisk (guestfs_h *g,
		const char *device,
		int cyls,
		int heads,
		int sectors,
		char *const *lines)
{
  struct guestfs_sfdisk_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdisk", "device");
    return -1;
  }
  if (lines == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdisk", "lines");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "sfdisk");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", cyls);
    fprintf (stderr, " %d", heads);
    fprintf (stderr, " %d", sectors);
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; lines[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (lines[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sfdisk") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.cyls = cyls;
  args.heads = heads;
  args.sectors = sectors;
  args.lines.lines_val = (char **) lines;
  for (args.lines.lines_len = 0; lines[args.lines.lines_len]; args.lines.lines_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_SFDISK,
        (xdrproc_t) xdr_guestfs_sfdisk_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "sfdisk", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SFDISK, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sfdisk", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_write_file (guestfs_h *g,
		const char *path,
		const char *content,
		int size)
{
  struct guestfs_write_file_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "write_file", "path");
    return -1;
  }
  if (content == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "write_file", "content");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "write_file");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " \"%s\"", content);
    fprintf (stderr, " %d", size);
    fputc ('\n', stderr);
  }

  if (check_state (g, "write_file") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.content = (char *) content;
  args.size = size;
  serial = guestfs___send (g, GUESTFS_PROC_WRITE_FILE,
        (xdrproc_t) xdr_guestfs_write_file_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "write_file", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_WRITE_FILE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "write_file", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_umount (guestfs_h *g,
		const char *pathordevice)
{
  struct guestfs_umount_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (pathordevice == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "umount", "pathordevice");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "umount");
    fprintf (stderr, " \"%s\"", pathordevice);
    fputc ('\n', stderr);
  }

  if (check_state (g, "umount") == -1) return -1;
  guestfs___set_busy (g);

  args.pathordevice = (char *) pathordevice;
  serial = guestfs___send (g, GUESTFS_PROC_UMOUNT,
        (xdrproc_t) xdr_guestfs_umount_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "umount", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_UMOUNT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "umount", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char **guestfs_mounts (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_mounts_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mounts");
    fputc ('\n', stderr);
  }

  if (check_state (g, "mounts") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_MOUNTS, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "mounts", &hdr, &err,
        (xdrproc_t) xdr_guestfs_mounts_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MOUNTS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mounts", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.devices.devices_val =
    safe_realloc (g, ret.devices.devices_val,
                  sizeof (char *) * (ret.devices.devices_len + 1));
  ret.devices.devices_val[ret.devices.devices_len] = NULL;
  return ret.devices.devices_val;
}

int guestfs_umount_all (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "umount_all");
    fputc ('\n', stderr);
  }

  if (check_state (g, "umount_all") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_UMOUNT_ALL, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "umount_all", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_UMOUNT_ALL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "umount_all", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_lvm_remove_all (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvm_remove_all");
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvm_remove_all") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_LVM_REMOVE_ALL, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lvm_remove_all", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVM_REMOVE_ALL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvm_remove_all", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_file (guestfs_h *g,
		const char *path)
{
  struct guestfs_file_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_file_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "file", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "file");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "file") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_FILE,
        (xdrproc_t) xdr_guestfs_file_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "file", &hdr, &err,
        (xdrproc_t) xdr_guestfs_file_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FILE, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "file", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.description; /* caller will free */
}

char *guestfs_command (guestfs_h *g,
		char *const *arguments)
{
  struct guestfs_command_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_command_ret ret;
  int serial;
  int r;

  if (arguments == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "command", "arguments");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "command");
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; arguments[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (arguments[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "command") == -1) return NULL;
  guestfs___set_busy (g);

  args.arguments.arguments_val = (char **) arguments;
  for (args.arguments.arguments_len = 0; arguments[args.arguments.arguments_len]; args.arguments.arguments_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_COMMAND,
        (xdrproc_t) xdr_guestfs_command_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "command", &hdr, &err,
        (xdrproc_t) xdr_guestfs_command_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_COMMAND, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "command", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.output; /* caller will free */
}

char **guestfs_command_lines (guestfs_h *g,
		char *const *arguments)
{
  struct guestfs_command_lines_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_command_lines_ret ret;
  int serial;
  int r;

  if (arguments == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "command_lines", "arguments");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "command_lines");
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; arguments[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (arguments[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "command_lines") == -1) return NULL;
  guestfs___set_busy (g);

  args.arguments.arguments_val = (char **) arguments;
  for (args.arguments.arguments_len = 0; arguments[args.arguments.arguments_len]; args.arguments.arguments_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_COMMAND_LINES,
        (xdrproc_t) xdr_guestfs_command_lines_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "command_lines", &hdr, &err,
        (xdrproc_t) xdr_guestfs_command_lines_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_COMMAND_LINES, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "command_lines", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

struct guestfs_stat *guestfs_stat (guestfs_h *g,
		const char *path)
{
  struct guestfs_stat_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_stat_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "stat", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "stat");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "stat") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_STAT,
        (xdrproc_t) xdr_guestfs_stat_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "stat", &hdr, &err,
        (xdrproc_t) xdr_guestfs_stat_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_STAT, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "stat", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.statbuf, sizeof (ret.statbuf));
}

struct guestfs_stat *guestfs_lstat (guestfs_h *g,
		const char *path)
{
  struct guestfs_lstat_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_lstat_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lstat", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lstat");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lstat") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_LSTAT,
        (xdrproc_t) xdr_guestfs_lstat_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "lstat", &hdr, &err,
        (xdrproc_t) xdr_guestfs_lstat_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LSTAT, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lstat", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.statbuf, sizeof (ret.statbuf));
}

struct guestfs_statvfs *guestfs_statvfs (guestfs_h *g,
		const char *path)
{
  struct guestfs_statvfs_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_statvfs_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "statvfs", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "statvfs");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "statvfs") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_STATVFS,
        (xdrproc_t) xdr_guestfs_statvfs_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "statvfs", &hdr, &err,
        (xdrproc_t) xdr_guestfs_statvfs_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_STATVFS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "statvfs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.statbuf, sizeof (ret.statbuf));
}

char **guestfs_tune2fs_l (guestfs_h *g,
		const char *device)
{
  struct guestfs_tune2fs_l_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_tune2fs_l_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tune2fs_l", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "tune2fs_l");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "tune2fs_l") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_TUNE2FS_L,
        (xdrproc_t) xdr_guestfs_tune2fs_l_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "tune2fs_l", &hdr, &err,
        (xdrproc_t) xdr_guestfs_tune2fs_l_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TUNE2FS_L, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "tune2fs_l", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.superblock.superblock_val =
    safe_realloc (g, ret.superblock.superblock_val,
                  sizeof (char *) * (ret.superblock.superblock_len + 1));
  ret.superblock.superblock_val[ret.superblock.superblock_len] = NULL;
  return ret.superblock.superblock_val;
}

int guestfs_blockdev_setro (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_setro_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_setro", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_setro");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_setro") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_SETRO,
        (xdrproc_t) xdr_guestfs_blockdev_setro_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "blockdev_setro", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_SETRO, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_setro", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_blockdev_setrw (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_setrw_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_setrw", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_setrw");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_setrw") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_SETRW,
        (xdrproc_t) xdr_guestfs_blockdev_setrw_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "blockdev_setrw", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_SETRW, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_setrw", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_blockdev_getro (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_getro_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_blockdev_getro_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_getro", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_getro");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_getro") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_GETRO,
        (xdrproc_t) xdr_guestfs_blockdev_getro_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "blockdev_getro", &hdr, &err,
        (xdrproc_t) xdr_guestfs_blockdev_getro_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_GETRO, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_getro", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.ro;
}

int guestfs_blockdev_getss (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_getss_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_blockdev_getss_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_getss", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_getss");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_getss") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_GETSS,
        (xdrproc_t) xdr_guestfs_blockdev_getss_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "blockdev_getss", &hdr, &err,
        (xdrproc_t) xdr_guestfs_blockdev_getss_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_GETSS, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_getss", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.sectorsize;
}

int guestfs_blockdev_getbsz (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_getbsz_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_blockdev_getbsz_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_getbsz", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_getbsz");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_getbsz") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_GETBSZ,
        (xdrproc_t) xdr_guestfs_blockdev_getbsz_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "blockdev_getbsz", &hdr, &err,
        (xdrproc_t) xdr_guestfs_blockdev_getbsz_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_GETBSZ, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_getbsz", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.blocksize;
}

int guestfs_blockdev_setbsz (guestfs_h *g,
		const char *device,
		int blocksize)
{
  struct guestfs_blockdev_setbsz_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_setbsz", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_setbsz");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", blocksize);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_setbsz") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.blocksize = blocksize;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_SETBSZ,
        (xdrproc_t) xdr_guestfs_blockdev_setbsz_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "blockdev_setbsz", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_SETBSZ, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_setbsz", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int64_t guestfs_blockdev_getsz (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_getsz_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_blockdev_getsz_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_getsz", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_getsz");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_getsz") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_GETSZ,
        (xdrproc_t) xdr_guestfs_blockdev_getsz_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "blockdev_getsz", &hdr, &err,
        (xdrproc_t) xdr_guestfs_blockdev_getsz_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_GETSZ, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_getsz", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.sizeinsectors;
}

int64_t guestfs_blockdev_getsize64 (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_getsize64_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_blockdev_getsize64_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_getsize64", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_getsize64");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_getsize64") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_GETSIZE64,
        (xdrproc_t) xdr_guestfs_blockdev_getsize64_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "blockdev_getsize64", &hdr, &err,
        (xdrproc_t) xdr_guestfs_blockdev_getsize64_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_GETSIZE64, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_getsize64", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.sizeinbytes;
}

int guestfs_blockdev_flushbufs (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_flushbufs_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_flushbufs", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_flushbufs");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_flushbufs") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_FLUSHBUFS,
        (xdrproc_t) xdr_guestfs_blockdev_flushbufs_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "blockdev_flushbufs", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_FLUSHBUFS, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_flushbufs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_blockdev_rereadpt (guestfs_h *g,
		const char *device)
{
  struct guestfs_blockdev_rereadpt_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "blockdev_rereadpt", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "blockdev_rereadpt");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "blockdev_rereadpt") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_BLOCKDEV_REREADPT,
        (xdrproc_t) xdr_guestfs_blockdev_rereadpt_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "blockdev_rereadpt", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BLOCKDEV_REREADPT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "blockdev_rereadpt", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_upload (guestfs_h *g,
		const char *filename,
		const char *remotefilename)
{
  struct guestfs_upload_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "upload", "filename");
    return -1;
  }
  if (remotefilename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "upload", "remotefilename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "upload");
    fprintf (stderr, " \"%s\"", filename);
    fprintf (stderr, " \"%s\"", remotefilename);
    fputc ('\n', stderr);
  }

  if (check_state (g, "upload") == -1) return -1;
  guestfs___set_busy (g);

  args.remotefilename = (char *) remotefilename;
  serial = guestfs___send (g, GUESTFS_PROC_UPLOAD,
        (xdrproc_t) xdr_guestfs_upload_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  r = guestfs___send_file (g, filename);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }
  if (r == -2) /* daemon cancelled */
    goto read_reply;

 read_reply:
  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "upload", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_UPLOAD, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "upload", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_download (guestfs_h *g,
		const char *remotefilename,
		const char *filename)
{
  struct guestfs_download_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (remotefilename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "download", "remotefilename");
    return -1;
  }
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "download", "filename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "download");
    fprintf (stderr, " \"%s\"", remotefilename);
    fprintf (stderr, " \"%s\"", filename);
    fputc ('\n', stderr);
  }

  if (check_state (g, "download") == -1) return -1;
  guestfs___set_busy (g);

  args.remotefilename = (char *) remotefilename;
  serial = guestfs___send (g, GUESTFS_PROC_DOWNLOAD,
        (xdrproc_t) xdr_guestfs_download_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "download", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DOWNLOAD, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "download", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  if (guestfs___recv_file (g, filename) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_checksum (guestfs_h *g,
		const char *csumtype,
		const char *path)
{
  struct guestfs_checksum_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_checksum_ret ret;
  int serial;
  int r;

  if (csumtype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "checksum", "csumtype");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "checksum", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "checksum");
    fprintf (stderr, " \"%s\"", csumtype);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "checksum") == -1) return NULL;
  guestfs___set_busy (g);

  args.csumtype = (char *) csumtype;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_CHECKSUM,
        (xdrproc_t) xdr_guestfs_checksum_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "checksum", &hdr, &err,
        (xdrproc_t) xdr_guestfs_checksum_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CHECKSUM, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "checksum", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.checksum; /* caller will free */
}

int guestfs_tar_in (guestfs_h *g,
		const char *tarfile,
		const char *directory)
{
  struct guestfs_tar_in_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (tarfile == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tar_in", "tarfile");
    return -1;
  }
  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tar_in", "directory");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "tar_in");
    fprintf (stderr, " \"%s\"", tarfile);
    fprintf (stderr, " \"%s\"", directory);
    fputc ('\n', stderr);
  }

  if (check_state (g, "tar_in") == -1) return -1;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_TAR_IN,
        (xdrproc_t) xdr_guestfs_tar_in_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  r = guestfs___send_file (g, tarfile);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }
  if (r == -2) /* daemon cancelled */
    goto read_reply;

 read_reply:
  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "tar_in", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TAR_IN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "tar_in", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_tar_out (guestfs_h *g,
		const char *directory,
		const char *tarfile)
{
  struct guestfs_tar_out_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tar_out", "directory");
    return -1;
  }
  if (tarfile == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tar_out", "tarfile");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "tar_out");
    fprintf (stderr, " \"%s\"", directory);
    fprintf (stderr, " \"%s\"", tarfile);
    fputc ('\n', stderr);
  }

  if (check_state (g, "tar_out") == -1) return -1;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_TAR_OUT,
        (xdrproc_t) xdr_guestfs_tar_out_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "tar_out", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TAR_OUT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "tar_out", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  if (guestfs___recv_file (g, tarfile) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_tgz_in (guestfs_h *g,
		const char *tarball,
		const char *directory)
{
  struct guestfs_tgz_in_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (tarball == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tgz_in", "tarball");
    return -1;
  }
  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tgz_in", "directory");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "tgz_in");
    fprintf (stderr, " \"%s\"", tarball);
    fprintf (stderr, " \"%s\"", directory);
    fputc ('\n', stderr);
  }

  if (check_state (g, "tgz_in") == -1) return -1;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_TGZ_IN,
        (xdrproc_t) xdr_guestfs_tgz_in_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  r = guestfs___send_file (g, tarball);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }
  if (r == -2) /* daemon cancelled */
    goto read_reply;

 read_reply:
  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "tgz_in", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TGZ_IN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "tgz_in", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_tgz_out (guestfs_h *g,
		const char *directory,
		const char *tarball)
{
  struct guestfs_tgz_out_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tgz_out", "directory");
    return -1;
  }
  if (tarball == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tgz_out", "tarball");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "tgz_out");
    fprintf (stderr, " \"%s\"", directory);
    fprintf (stderr, " \"%s\"", tarball);
    fputc ('\n', stderr);
  }

  if (check_state (g, "tgz_out") == -1) return -1;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_TGZ_OUT,
        (xdrproc_t) xdr_guestfs_tgz_out_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "tgz_out", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TGZ_OUT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "tgz_out", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  if (guestfs___recv_file (g, tarball) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mount_ro (guestfs_h *g,
		const char *device,
		const char *mountpoint)
{
  struct guestfs_mount_ro_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_ro", "device");
    return -1;
  }
  if (mountpoint == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_ro", "mountpoint");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mount_ro");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", mountpoint);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mount_ro") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.mountpoint = (char *) mountpoint;
  serial = guestfs___send (g, GUESTFS_PROC_MOUNT_RO,
        (xdrproc_t) xdr_guestfs_mount_ro_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mount_ro", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MOUNT_RO, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mount_ro", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mount_options (guestfs_h *g,
		const char *options,
		const char *device,
		const char *mountpoint)
{
  struct guestfs_mount_options_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (options == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_options", "options");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_options", "device");
    return -1;
  }
  if (mountpoint == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_options", "mountpoint");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mount_options");
    fprintf (stderr, " \"%s\"", options);
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", mountpoint);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mount_options") == -1) return -1;
  guestfs___set_busy (g);

  args.options = (char *) options;
  args.device = (char *) device;
  args.mountpoint = (char *) mountpoint;
  serial = guestfs___send (g, GUESTFS_PROC_MOUNT_OPTIONS,
        (xdrproc_t) xdr_guestfs_mount_options_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mount_options", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MOUNT_OPTIONS, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mount_options", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mount_vfs (guestfs_h *g,
		const char *options,
		const char *vfstype,
		const char *device,
		const char *mountpoint)
{
  struct guestfs_mount_vfs_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (options == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_vfs", "options");
    return -1;
  }
  if (vfstype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_vfs", "vfstype");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_vfs", "device");
    return -1;
  }
  if (mountpoint == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_vfs", "mountpoint");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mount_vfs");
    fprintf (stderr, " \"%s\"", options);
    fprintf (stderr, " \"%s\"", vfstype);
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", mountpoint);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mount_vfs") == -1) return -1;
  guestfs___set_busy (g);

  args.options = (char *) options;
  args.vfstype = (char *) vfstype;
  args.device = (char *) device;
  args.mountpoint = (char *) mountpoint;
  serial = guestfs___send (g, GUESTFS_PROC_MOUNT_VFS,
        (xdrproc_t) xdr_guestfs_mount_vfs_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mount_vfs", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MOUNT_VFS, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mount_vfs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_debug (guestfs_h *g,
		const char *subcmd,
		char *const *extraargs)
{
  struct guestfs_debug_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_debug_ret ret;
  int serial;
  int r;

  if (subcmd == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "debug", "subcmd");
    return NULL;
  }
  if (extraargs == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "debug", "extraargs");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "debug");
    fprintf (stderr, " \"%s\"", subcmd);
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; extraargs[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (extraargs[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "debug") == -1) return NULL;
  guestfs___set_busy (g);

  args.subcmd = (char *) subcmd;
  args.extraargs.extraargs_val = (char **) extraargs;
  for (args.extraargs.extraargs_len = 0; extraargs[args.extraargs.extraargs_len]; args.extraargs.extraargs_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_DEBUG,
        (xdrproc_t) xdr_guestfs_debug_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "debug", &hdr, &err,
        (xdrproc_t) xdr_guestfs_debug_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DEBUG, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "debug", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.result; /* caller will free */
}

int guestfs_lvremove (guestfs_h *g,
		const char *device)
{
  struct guestfs_lvremove_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvremove", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvremove");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvremove") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_LVREMOVE,
        (xdrproc_t) xdr_guestfs_lvremove_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lvremove", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVREMOVE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvremove", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_vgremove (guestfs_h *g,
		const char *vgname)
{
  struct guestfs_vgremove_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (vgname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vgremove", "vgname");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vgremove");
    fprintf (stderr, " \"%s\"", vgname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vgremove") == -1) return -1;
  guestfs___set_busy (g);

  args.vgname = (char *) vgname;
  serial = guestfs___send (g, GUESTFS_PROC_VGREMOVE,
        (xdrproc_t) xdr_guestfs_vgremove_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "vgremove", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGREMOVE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vgremove", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_pvremove (guestfs_h *g,
		const char *device)
{
  struct guestfs_pvremove_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pvremove", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pvremove");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pvremove") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_PVREMOVE,
        (xdrproc_t) xdr_guestfs_pvremove_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "pvremove", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PVREMOVE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pvremove", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_set_e2label (guestfs_h *g,
		const char *device,
		const char *label)
{
  struct guestfs_set_e2label_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "set_e2label", "device");
    return -1;
  }
  if (label == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "set_e2label", "label");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_e2label");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", label);
    fputc ('\n', stderr);
  }

  if (check_state (g, "set_e2label") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.label = (char *) label;
  serial = guestfs___send (g, GUESTFS_PROC_SET_E2LABEL,
        (xdrproc_t) xdr_guestfs_set_e2label_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "set_e2label", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SET_E2LABEL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "set_e2label", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_get_e2label (guestfs_h *g,
		const char *device)
{
  struct guestfs_get_e2label_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_get_e2label_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "get_e2label", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_e2label");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "get_e2label") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_GET_E2LABEL,
        (xdrproc_t) xdr_guestfs_get_e2label_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "get_e2label", &hdr, &err,
        (xdrproc_t) xdr_guestfs_get_e2label_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GET_E2LABEL, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "get_e2label", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.label; /* caller will free */
}

int guestfs_set_e2uuid (guestfs_h *g,
		const char *device,
		const char *uuid)
{
  struct guestfs_set_e2uuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "set_e2uuid", "device");
    return -1;
  }
  if (uuid == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "set_e2uuid", "uuid");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "set_e2uuid");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", uuid);
    fputc ('\n', stderr);
  }

  if (check_state (g, "set_e2uuid") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.uuid = (char *) uuid;
  serial = guestfs___send (g, GUESTFS_PROC_SET_E2UUID,
        (xdrproc_t) xdr_guestfs_set_e2uuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "set_e2uuid", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SET_E2UUID, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "set_e2uuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_get_e2uuid (guestfs_h *g,
		const char *device)
{
  struct guestfs_get_e2uuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_get_e2uuid_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "get_e2uuid", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_e2uuid");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "get_e2uuid") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_GET_E2UUID,
        (xdrproc_t) xdr_guestfs_get_e2uuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "get_e2uuid", &hdr, &err,
        (xdrproc_t) xdr_guestfs_get_e2uuid_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GET_E2UUID, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "get_e2uuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.uuid; /* caller will free */
}

int guestfs_fsck (guestfs_h *g,
		const char *fstype,
		const char *device)
{
  struct guestfs_fsck_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_fsck_ret ret;
  int serial;
  int r;

  if (fstype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fsck", "fstype");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fsck", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "fsck");
    fprintf (stderr, " \"%s\"", fstype);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "fsck") == -1) return -1;
  guestfs___set_busy (g);

  args.fstype = (char *) fstype;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_FSCK,
        (xdrproc_t) xdr_guestfs_fsck_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "fsck", &hdr, &err,
        (xdrproc_t) xdr_guestfs_fsck_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FSCK, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "fsck", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.status;
}

int guestfs_zero (guestfs_h *g,
		const char *device)
{
  struct guestfs_zero_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zero", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zero");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zero") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_ZERO,
        (xdrproc_t) xdr_guestfs_zero_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "zero", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZERO, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zero", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_grub_install (guestfs_h *g,
		const char *root,
		const char *device)
{
  struct guestfs_grub_install_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (root == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "grub_install", "root");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "grub_install", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "grub_install");
    fprintf (stderr, " \"%s\"", root);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "grub_install") == -1) return -1;
  guestfs___set_busy (g);

  args.root = (char *) root;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_GRUB_INSTALL,
        (xdrproc_t) xdr_guestfs_grub_install_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "grub_install", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GRUB_INSTALL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "grub_install", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_cp (guestfs_h *g,
		const char *src,
		const char *dest)
{
  struct guestfs_cp_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (src == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "cp", "src");
    return -1;
  }
  if (dest == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "cp", "dest");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "cp");
    fprintf (stderr, " \"%s\"", src);
    fprintf (stderr, " \"%s\"", dest);
    fputc ('\n', stderr);
  }

  if (check_state (g, "cp") == -1) return -1;
  guestfs___set_busy (g);

  args.src = (char *) src;
  args.dest = (char *) dest;
  serial = guestfs___send (g, GUESTFS_PROC_CP,
        (xdrproc_t) xdr_guestfs_cp_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "cp", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CP, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "cp", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_cp_a (guestfs_h *g,
		const char *src,
		const char *dest)
{
  struct guestfs_cp_a_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (src == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "cp_a", "src");
    return -1;
  }
  if (dest == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "cp_a", "dest");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "cp_a");
    fprintf (stderr, " \"%s\"", src);
    fprintf (stderr, " \"%s\"", dest);
    fputc ('\n', stderr);
  }

  if (check_state (g, "cp_a") == -1) return -1;
  guestfs___set_busy (g);

  args.src = (char *) src;
  args.dest = (char *) dest;
  serial = guestfs___send (g, GUESTFS_PROC_CP_A,
        (xdrproc_t) xdr_guestfs_cp_a_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "cp_a", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CP_A, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "cp_a", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mv (guestfs_h *g,
		const char *src,
		const char *dest)
{
  struct guestfs_mv_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (src == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mv", "src");
    return -1;
  }
  if (dest == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mv", "dest");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mv");
    fprintf (stderr, " \"%s\"", src);
    fprintf (stderr, " \"%s\"", dest);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mv") == -1) return -1;
  guestfs___set_busy (g);

  args.src = (char *) src;
  args.dest = (char *) dest;
  serial = guestfs___send (g, GUESTFS_PROC_MV,
        (xdrproc_t) xdr_guestfs_mv_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mv", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MV, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mv", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_drop_caches (guestfs_h *g,
		int whattodrop)
{
  struct guestfs_drop_caches_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "drop_caches");
    fprintf (stderr, " %d", whattodrop);
    fputc ('\n', stderr);
  }

  if (check_state (g, "drop_caches") == -1) return -1;
  guestfs___set_busy (g);

  args.whattodrop = whattodrop;
  serial = guestfs___send (g, GUESTFS_PROC_DROP_CACHES,
        (xdrproc_t) xdr_guestfs_drop_caches_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "drop_caches", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DROP_CACHES, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "drop_caches", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_dmesg (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_dmesg_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "dmesg");
    fputc ('\n', stderr);
  }

  if (check_state (g, "dmesg") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_DMESG, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "dmesg", &hdr, &err,
        (xdrproc_t) xdr_guestfs_dmesg_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DMESG, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "dmesg", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.kmsgs; /* caller will free */
}

int guestfs_ping_daemon (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ping_daemon");
    fputc ('\n', stderr);
  }

  if (check_state (g, "ping_daemon") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_PING_DAEMON, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "ping_daemon", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PING_DAEMON, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ping_daemon", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_equal (guestfs_h *g,
		const char *file1,
		const char *file2)
{
  struct guestfs_equal_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_equal_ret ret;
  int serial;
  int r;

  if (file1 == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "equal", "file1");
    return -1;
  }
  if (file2 == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "equal", "file2");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "equal");
    fprintf (stderr, " \"%s\"", file1);
    fprintf (stderr, " \"%s\"", file2);
    fputc ('\n', stderr);
  }

  if (check_state (g, "equal") == -1) return -1;
  guestfs___set_busy (g);

  args.file1 = (char *) file1;
  args.file2 = (char *) file2;
  serial = guestfs___send (g, GUESTFS_PROC_EQUAL,
        (xdrproc_t) xdr_guestfs_equal_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "equal", &hdr, &err,
        (xdrproc_t) xdr_guestfs_equal_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_EQUAL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "equal", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.equality;
}

char **guestfs_strings (guestfs_h *g,
		const char *path)
{
  struct guestfs_strings_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_strings_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "strings", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "strings");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "strings") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_STRINGS,
        (xdrproc_t) xdr_guestfs_strings_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "strings", &hdr, &err,
        (xdrproc_t) xdr_guestfs_strings_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_STRINGS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "strings", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.stringsout.stringsout_val =
    safe_realloc (g, ret.stringsout.stringsout_val,
                  sizeof (char *) * (ret.stringsout.stringsout_len + 1));
  ret.stringsout.stringsout_val[ret.stringsout.stringsout_len] = NULL;
  return ret.stringsout.stringsout_val;
}

char **guestfs_strings_e (guestfs_h *g,
		const char *encoding,
		const char *path)
{
  struct guestfs_strings_e_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_strings_e_ret ret;
  int serial;
  int r;

  if (encoding == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "strings_e", "encoding");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "strings_e", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "strings_e");
    fprintf (stderr, " \"%s\"", encoding);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "strings_e") == -1) return NULL;
  guestfs___set_busy (g);

  args.encoding = (char *) encoding;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_STRINGS_E,
        (xdrproc_t) xdr_guestfs_strings_e_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "strings_e", &hdr, &err,
        (xdrproc_t) xdr_guestfs_strings_e_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_STRINGS_E, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "strings_e", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.stringsout.stringsout_val =
    safe_realloc (g, ret.stringsout.stringsout_val,
                  sizeof (char *) * (ret.stringsout.stringsout_len + 1));
  ret.stringsout.stringsout_val[ret.stringsout.stringsout_len] = NULL;
  return ret.stringsout.stringsout_val;
}

char *guestfs_hexdump (guestfs_h *g,
		const char *path)
{
  struct guestfs_hexdump_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_hexdump_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "hexdump", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "hexdump");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "hexdump") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_HEXDUMP,
        (xdrproc_t) xdr_guestfs_hexdump_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "hexdump", &hdr, &err,
        (xdrproc_t) xdr_guestfs_hexdump_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_HEXDUMP, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "hexdump", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.dump; /* caller will free */
}

int guestfs_zerofree (guestfs_h *g,
		const char *device)
{
  struct guestfs_zerofree_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zerofree", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zerofree");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zerofree") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_ZEROFREE,
        (xdrproc_t) xdr_guestfs_zerofree_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "zerofree", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZEROFREE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zerofree", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_pvresize (guestfs_h *g,
		const char *device)
{
  struct guestfs_pvresize_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pvresize", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pvresize");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pvresize") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_PVRESIZE,
        (xdrproc_t) xdr_guestfs_pvresize_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "pvresize", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PVRESIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pvresize", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_sfdisk_N (guestfs_h *g,
		const char *device,
		int partnum,
		int cyls,
		int heads,
		int sectors,
		const char *line)
{
  struct guestfs_sfdisk_N_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdisk_N", "device");
    return -1;
  }
  if (line == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdisk_N", "line");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "sfdisk_N");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", partnum);
    fprintf (stderr, " %d", cyls);
    fprintf (stderr, " %d", heads);
    fprintf (stderr, " %d", sectors);
    fprintf (stderr, " \"%s\"", line);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sfdisk_N") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.partnum = partnum;
  args.cyls = cyls;
  args.heads = heads;
  args.sectors = sectors;
  args.line = (char *) line;
  serial = guestfs___send (g, GUESTFS_PROC_SFDISK_N,
        (xdrproc_t) xdr_guestfs_sfdisk_N_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "sfdisk_N", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SFDISK_N, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sfdisk_N", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_sfdisk_l (guestfs_h *g,
		const char *device)
{
  struct guestfs_sfdisk_l_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_sfdisk_l_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdisk_l", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "sfdisk_l");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sfdisk_l") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_SFDISK_L,
        (xdrproc_t) xdr_guestfs_sfdisk_l_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "sfdisk_l", &hdr, &err,
        (xdrproc_t) xdr_guestfs_sfdisk_l_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SFDISK_L, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sfdisk_l", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.partitions; /* caller will free */
}

char *guestfs_sfdisk_kernel_geometry (guestfs_h *g,
		const char *device)
{
  struct guestfs_sfdisk_kernel_geometry_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_sfdisk_kernel_geometry_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdisk_kernel_geometry", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "sfdisk_kernel_geometry");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sfdisk_kernel_geometry") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_SFDISK_KERNEL_GEOMETRY,
        (xdrproc_t) xdr_guestfs_sfdisk_kernel_geometry_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "sfdisk_kernel_geometry", &hdr, &err,
        (xdrproc_t) xdr_guestfs_sfdisk_kernel_geometry_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SFDISK_KERNEL_GEOMETRY, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sfdisk_kernel_geometry", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.partitions; /* caller will free */
}

char *guestfs_sfdisk_disk_geometry (guestfs_h *g,
		const char *device)
{
  struct guestfs_sfdisk_disk_geometry_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_sfdisk_disk_geometry_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdisk_disk_geometry", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "sfdisk_disk_geometry");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sfdisk_disk_geometry") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_SFDISK_DISK_GEOMETRY,
        (xdrproc_t) xdr_guestfs_sfdisk_disk_geometry_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "sfdisk_disk_geometry", &hdr, &err,
        (xdrproc_t) xdr_guestfs_sfdisk_disk_geometry_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SFDISK_DISK_GEOMETRY, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sfdisk_disk_geometry", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.partitions; /* caller will free */
}

int guestfs_vg_activate_all (guestfs_h *g,
		int activate)
{
  struct guestfs_vg_activate_all_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vg_activate_all");
    fputs (activate ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vg_activate_all") == -1) return -1;
  guestfs___set_busy (g);

  args.activate = activate;
  serial = guestfs___send (g, GUESTFS_PROC_VG_ACTIVATE_ALL,
        (xdrproc_t) xdr_guestfs_vg_activate_all_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "vg_activate_all", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VG_ACTIVATE_ALL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vg_activate_all", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_vg_activate (guestfs_h *g,
		int activate,
		char *const *volgroups)
{
  struct guestfs_vg_activate_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (volgroups == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vg_activate", "volgroups");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "vg_activate");
    fputs (activate ? " true" : " false", stderr);
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; volgroups[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (volgroups[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vg_activate") == -1) return -1;
  guestfs___set_busy (g);

  args.activate = activate;
  args.volgroups.volgroups_val = (char **) volgroups;
  for (args.volgroups.volgroups_len = 0; volgroups[args.volgroups.volgroups_len]; args.volgroups.volgroups_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_VG_ACTIVATE,
        (xdrproc_t) xdr_guestfs_vg_activate_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "vg_activate", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VG_ACTIVATE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vg_activate", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_lvresize (guestfs_h *g,
		const char *device,
		int mbytes)
{
  struct guestfs_lvresize_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvresize", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvresize");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", mbytes);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvresize") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.mbytes = mbytes;
  serial = guestfs___send (g, GUESTFS_PROC_LVRESIZE,
        (xdrproc_t) xdr_guestfs_lvresize_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lvresize", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVRESIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvresize", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_resize2fs (guestfs_h *g,
		const char *device)
{
  struct guestfs_resize2fs_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "resize2fs", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "resize2fs");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "resize2fs") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_RESIZE2FS,
        (xdrproc_t) xdr_guestfs_resize2fs_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "resize2fs", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_RESIZE2FS, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "resize2fs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char **guestfs_find (guestfs_h *g,
		const char *directory)
{
  struct guestfs_find_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_find_ret ret;
  int serial;
  int r;

  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "find", "directory");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "find");
    fprintf (stderr, " \"%s\"", directory);
    fputc ('\n', stderr);
  }

  if (check_state (g, "find") == -1) return NULL;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_FIND,
        (xdrproc_t) xdr_guestfs_find_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "find", &hdr, &err,
        (xdrproc_t) xdr_guestfs_find_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FIND, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "find", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.names.names_val =
    safe_realloc (g, ret.names.names_val,
                  sizeof (char *) * (ret.names.names_len + 1));
  ret.names.names_val[ret.names.names_len] = NULL;
  return ret.names.names_val;
}

int guestfs_e2fsck_f (guestfs_h *g,
		const char *device)
{
  struct guestfs_e2fsck_f_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "e2fsck_f", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "e2fsck_f");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "e2fsck_f") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_E2FSCK_F,
        (xdrproc_t) xdr_guestfs_e2fsck_f_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "e2fsck_f", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_E2FSCK_F, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "e2fsck_f", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_sleep (guestfs_h *g,
		int secs)
{
  struct guestfs_sleep_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "sleep");
    fprintf (stderr, " %d", secs);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sleep") == -1) return -1;
  guestfs___set_busy (g);

  args.secs = secs;
  serial = guestfs___send (g, GUESTFS_PROC_SLEEP,
        (xdrproc_t) xdr_guestfs_sleep_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "sleep", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SLEEP, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sleep", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_ntfs_3g_probe (guestfs_h *g,
		int rw,
		const char *device)
{
  struct guestfs_ntfs_3g_probe_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_ntfs_3g_probe_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ntfs_3g_probe", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ntfs_3g_probe");
    fputs (rw ? " true" : " false", stderr);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ntfs_3g_probe") == -1) return -1;
  guestfs___set_busy (g);

  args.rw = rw;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_NTFS_3G_PROBE,
        (xdrproc_t) xdr_guestfs_ntfs_3g_probe_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "ntfs_3g_probe", &hdr, &err,
        (xdrproc_t) xdr_guestfs_ntfs_3g_probe_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_NTFS_3G_PROBE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ntfs_3g_probe", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.status;
}

char *guestfs_sh (guestfs_h *g,
		const char *command)
{
  struct guestfs_sh_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_sh_ret ret;
  int serial;
  int r;

  if (command == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sh", "command");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "sh");
    fprintf (stderr, " \"%s\"", command);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sh") == -1) return NULL;
  guestfs___set_busy (g);

  args.command = (char *) command;
  serial = guestfs___send (g, GUESTFS_PROC_SH,
        (xdrproc_t) xdr_guestfs_sh_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "sh", &hdr, &err,
        (xdrproc_t) xdr_guestfs_sh_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SH, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sh", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.output; /* caller will free */
}

char **guestfs_sh_lines (guestfs_h *g,
		const char *command)
{
  struct guestfs_sh_lines_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_sh_lines_ret ret;
  int serial;
  int r;

  if (command == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sh_lines", "command");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "sh_lines");
    fprintf (stderr, " \"%s\"", command);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sh_lines") == -1) return NULL;
  guestfs___set_busy (g);

  args.command = (char *) command;
  serial = guestfs___send (g, GUESTFS_PROC_SH_LINES,
        (xdrproc_t) xdr_guestfs_sh_lines_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "sh_lines", &hdr, &err,
        (xdrproc_t) xdr_guestfs_sh_lines_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SH_LINES, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sh_lines", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_glob_expand (guestfs_h *g,
		const char *pattern)
{
  struct guestfs_glob_expand_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_glob_expand_ret ret;
  int serial;
  int r;

  if (pattern == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "glob_expand", "pattern");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "glob_expand");
    fprintf (stderr, " \"%s\"", pattern);
    fputc ('\n', stderr);
  }

  if (check_state (g, "glob_expand") == -1) return NULL;
  guestfs___set_busy (g);

  args.pattern = (char *) pattern;
  serial = guestfs___send (g, GUESTFS_PROC_GLOB_EXPAND,
        (xdrproc_t) xdr_guestfs_glob_expand_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "glob_expand", &hdr, &err,
        (xdrproc_t) xdr_guestfs_glob_expand_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GLOB_EXPAND, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "glob_expand", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.paths.paths_val =
    safe_realloc (g, ret.paths.paths_val,
                  sizeof (char *) * (ret.paths.paths_len + 1));
  ret.paths.paths_val[ret.paths.paths_len] = NULL;
  return ret.paths.paths_val;
}

int guestfs_scrub_device (guestfs_h *g,
		const char *device)
{
  struct guestfs_scrub_device_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "scrub_device", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "scrub_device");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "scrub_device") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_SCRUB_DEVICE,
        (xdrproc_t) xdr_guestfs_scrub_device_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "scrub_device", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SCRUB_DEVICE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "scrub_device", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_scrub_file (guestfs_h *g,
		const char *file)
{
  struct guestfs_scrub_file_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (file == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "scrub_file", "file");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "scrub_file");
    fprintf (stderr, " \"%s\"", file);
    fputc ('\n', stderr);
  }

  if (check_state (g, "scrub_file") == -1) return -1;
  guestfs___set_busy (g);

  args.file = (char *) file;
  serial = guestfs___send (g, GUESTFS_PROC_SCRUB_FILE,
        (xdrproc_t) xdr_guestfs_scrub_file_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "scrub_file", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SCRUB_FILE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "scrub_file", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_scrub_freespace (guestfs_h *g,
		const char *dir)
{
  struct guestfs_scrub_freespace_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (dir == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "scrub_freespace", "dir");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "scrub_freespace");
    fprintf (stderr, " \"%s\"", dir);
    fputc ('\n', stderr);
  }

  if (check_state (g, "scrub_freespace") == -1) return -1;
  guestfs___set_busy (g);

  args.dir = (char *) dir;
  serial = guestfs___send (g, GUESTFS_PROC_SCRUB_FREESPACE,
        (xdrproc_t) xdr_guestfs_scrub_freespace_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "scrub_freespace", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SCRUB_FREESPACE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "scrub_freespace", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_mkdtemp (guestfs_h *g,
		const char *template)
{
  struct guestfs_mkdtemp_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_mkdtemp_ret ret;
  int serial;
  int r;

  if (template == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkdtemp", "template");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkdtemp");
    fprintf (stderr, " \"%s\"", template);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkdtemp") == -1) return NULL;
  guestfs___set_busy (g);

  args.template = (char *) template;
  serial = guestfs___send (g, GUESTFS_PROC_MKDTEMP,
        (xdrproc_t) xdr_guestfs_mkdtemp_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "mkdtemp", &hdr, &err,
        (xdrproc_t) xdr_guestfs_mkdtemp_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKDTEMP, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkdtemp", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.dir; /* caller will free */
}

int guestfs_wc_l (guestfs_h *g,
		const char *path)
{
  struct guestfs_wc_l_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_wc_l_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "wc_l", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "wc_l");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "wc_l") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_WC_L,
        (xdrproc_t) xdr_guestfs_wc_l_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "wc_l", &hdr, &err,
        (xdrproc_t) xdr_guestfs_wc_l_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_WC_L, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "wc_l", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.lines;
}

int guestfs_wc_w (guestfs_h *g,
		const char *path)
{
  struct guestfs_wc_w_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_wc_w_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "wc_w", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "wc_w");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "wc_w") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_WC_W,
        (xdrproc_t) xdr_guestfs_wc_w_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "wc_w", &hdr, &err,
        (xdrproc_t) xdr_guestfs_wc_w_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_WC_W, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "wc_w", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.words;
}

int guestfs_wc_c (guestfs_h *g,
		const char *path)
{
  struct guestfs_wc_c_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_wc_c_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "wc_c", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "wc_c");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "wc_c") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_WC_C,
        (xdrproc_t) xdr_guestfs_wc_c_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "wc_c", &hdr, &err,
        (xdrproc_t) xdr_guestfs_wc_c_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_WC_C, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "wc_c", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.chars;
}

char **guestfs_head (guestfs_h *g,
		const char *path)
{
  struct guestfs_head_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_head_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "head", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "head");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "head") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_HEAD,
        (xdrproc_t) xdr_guestfs_head_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "head", &hdr, &err,
        (xdrproc_t) xdr_guestfs_head_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_HEAD, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "head", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_head_n (guestfs_h *g,
		int nrlines,
		const char *path)
{
  struct guestfs_head_n_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_head_n_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "head_n", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "head_n");
    fprintf (stderr, " %d", nrlines);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "head_n") == -1) return NULL;
  guestfs___set_busy (g);

  args.nrlines = nrlines;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_HEAD_N,
        (xdrproc_t) xdr_guestfs_head_n_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "head_n", &hdr, &err,
        (xdrproc_t) xdr_guestfs_head_n_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_HEAD_N, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "head_n", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_tail (guestfs_h *g,
		const char *path)
{
  struct guestfs_tail_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_tail_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tail", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "tail");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "tail") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_TAIL,
        (xdrproc_t) xdr_guestfs_tail_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "tail", &hdr, &err,
        (xdrproc_t) xdr_guestfs_tail_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TAIL, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "tail", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_tail_n (guestfs_h *g,
		int nrlines,
		const char *path)
{
  struct guestfs_tail_n_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_tail_n_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "tail_n", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "tail_n");
    fprintf (stderr, " %d", nrlines);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "tail_n") == -1) return NULL;
  guestfs___set_busy (g);

  args.nrlines = nrlines;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_TAIL_N,
        (xdrproc_t) xdr_guestfs_tail_n_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "tail_n", &hdr, &err,
        (xdrproc_t) xdr_guestfs_tail_n_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TAIL_N, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "tail_n", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char *guestfs_df (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_df_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "df");
    fputc ('\n', stderr);
  }

  if (check_state (g, "df") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_DF, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "df", &hdr, &err,
        (xdrproc_t) xdr_guestfs_df_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DF, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "df", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.output; /* caller will free */
}

char *guestfs_df_h (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_df_h_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "df_h");
    fputc ('\n', stderr);
  }

  if (check_state (g, "df_h") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_DF_H, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "df_h", &hdr, &err,
        (xdrproc_t) xdr_guestfs_df_h_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DF_H, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "df_h", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.output; /* caller will free */
}

int64_t guestfs_du (guestfs_h *g,
		const char *path)
{
  struct guestfs_du_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_du_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "du", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "du");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "du") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_DU,
        (xdrproc_t) xdr_guestfs_du_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "du", &hdr, &err,
        (xdrproc_t) xdr_guestfs_du_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DU, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "du", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.sizekb;
}

char **guestfs_initrd_list (guestfs_h *g,
		const char *path)
{
  struct guestfs_initrd_list_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_initrd_list_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "initrd_list", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "initrd_list");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "initrd_list") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_INITRD_LIST,
        (xdrproc_t) xdr_guestfs_initrd_list_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "initrd_list", &hdr, &err,
        (xdrproc_t) xdr_guestfs_initrd_list_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_INITRD_LIST, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "initrd_list", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.filenames.filenames_val =
    safe_realloc (g, ret.filenames.filenames_val,
                  sizeof (char *) * (ret.filenames.filenames_len + 1));
  ret.filenames.filenames_val[ret.filenames.filenames_len] = NULL;
  return ret.filenames.filenames_val;
}

int guestfs_mount_loop (guestfs_h *g,
		const char *file,
		const char *mountpoint)
{
  struct guestfs_mount_loop_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (file == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_loop", "file");
    return -1;
  }
  if (mountpoint == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mount_loop", "mountpoint");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mount_loop");
    fprintf (stderr, " \"%s\"", file);
    fprintf (stderr, " \"%s\"", mountpoint);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mount_loop") == -1) return -1;
  guestfs___set_busy (g);

  args.file = (char *) file;
  args.mountpoint = (char *) mountpoint;
  serial = guestfs___send (g, GUESTFS_PROC_MOUNT_LOOP,
        (xdrproc_t) xdr_guestfs_mount_loop_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mount_loop", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MOUNT_LOOP, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mount_loop", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkswap (guestfs_h *g,
		const char *device)
{
  struct guestfs_mkswap_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkswap", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkswap");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkswap") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_MKSWAP,
        (xdrproc_t) xdr_guestfs_mkswap_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkswap", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKSWAP, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkswap", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkswap_L (guestfs_h *g,
		const char *label,
		const char *device)
{
  struct guestfs_mkswap_L_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (label == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkswap_L", "label");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkswap_L", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkswap_L");
    fprintf (stderr, " \"%s\"", label);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkswap_L") == -1) return -1;
  guestfs___set_busy (g);

  args.label = (char *) label;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_MKSWAP_L,
        (xdrproc_t) xdr_guestfs_mkswap_L_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkswap_L", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKSWAP_L, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkswap_L", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkswap_U (guestfs_h *g,
		const char *uuid,
		const char *device)
{
  struct guestfs_mkswap_U_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (uuid == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkswap_U", "uuid");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkswap_U", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkswap_U");
    fprintf (stderr, " \"%s\"", uuid);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkswap_U") == -1) return -1;
  guestfs___set_busy (g);

  args.uuid = (char *) uuid;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_MKSWAP_U,
        (xdrproc_t) xdr_guestfs_mkswap_U_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkswap_U", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKSWAP_U, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkswap_U", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mknod (guestfs_h *g,
		int mode,
		int devmajor,
		int devminor,
		const char *path)
{
  struct guestfs_mknod_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mknod", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mknod");
    fprintf (stderr, " %d", mode);
    fprintf (stderr, " %d", devmajor);
    fprintf (stderr, " %d", devminor);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mknod") == -1) return -1;
  guestfs___set_busy (g);

  args.mode = mode;
  args.devmajor = devmajor;
  args.devminor = devminor;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_MKNOD,
        (xdrproc_t) xdr_guestfs_mknod_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mknod", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKNOD, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mknod", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkfifo (guestfs_h *g,
		int mode,
		const char *path)
{
  struct guestfs_mkfifo_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkfifo", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkfifo");
    fprintf (stderr, " %d", mode);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkfifo") == -1) return -1;
  guestfs___set_busy (g);

  args.mode = mode;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_MKFIFO,
        (xdrproc_t) xdr_guestfs_mkfifo_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkfifo", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKFIFO, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkfifo", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mknod_b (guestfs_h *g,
		int mode,
		int devmajor,
		int devminor,
		const char *path)
{
  struct guestfs_mknod_b_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mknod_b", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mknod_b");
    fprintf (stderr, " %d", mode);
    fprintf (stderr, " %d", devmajor);
    fprintf (stderr, " %d", devminor);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mknod_b") == -1) return -1;
  guestfs___set_busy (g);

  args.mode = mode;
  args.devmajor = devmajor;
  args.devminor = devminor;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_MKNOD_B,
        (xdrproc_t) xdr_guestfs_mknod_b_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mknod_b", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKNOD_B, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mknod_b", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mknod_c (guestfs_h *g,
		int mode,
		int devmajor,
		int devminor,
		const char *path)
{
  struct guestfs_mknod_c_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mknod_c", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mknod_c");
    fprintf (stderr, " %d", mode);
    fprintf (stderr, " %d", devmajor);
    fprintf (stderr, " %d", devminor);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mknod_c") == -1) return -1;
  guestfs___set_busy (g);

  args.mode = mode;
  args.devmajor = devmajor;
  args.devminor = devminor;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_MKNOD_C,
        (xdrproc_t) xdr_guestfs_mknod_c_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mknod_c", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKNOD_C, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mknod_c", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_umask (guestfs_h *g,
		int mask)
{
  struct guestfs_umask_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_umask_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "umask");
    fprintf (stderr, " %d", mask);
    fputc ('\n', stderr);
  }

  if (check_state (g, "umask") == -1) return -1;
  guestfs___set_busy (g);

  args.mask = mask;
  serial = guestfs___send (g, GUESTFS_PROC_UMASK,
        (xdrproc_t) xdr_guestfs_umask_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "umask", &hdr, &err,
        (xdrproc_t) xdr_guestfs_umask_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_UMASK, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "umask", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.oldmask;
}

struct guestfs_dirent_list *guestfs_readdir (guestfs_h *g,
		const char *dir)
{
  struct guestfs_readdir_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_readdir_ret ret;
  int serial;
  int r;

  if (dir == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "readdir", "dir");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "readdir");
    fprintf (stderr, " \"%s\"", dir);
    fputc ('\n', stderr);
  }

  if (check_state (g, "readdir") == -1) return NULL;
  guestfs___set_busy (g);

  args.dir = (char *) dir;
  serial = guestfs___send (g, GUESTFS_PROC_READDIR,
        (xdrproc_t) xdr_guestfs_readdir_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "readdir", &hdr, &err,
        (xdrproc_t) xdr_guestfs_readdir_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_READDIR, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "readdir", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.entries, sizeof (ret.entries));
}

int guestfs_sfdiskM (guestfs_h *g,
		const char *device,
		char *const *lines)
{
  struct guestfs_sfdiskM_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdiskM", "device");
    return -1;
  }
  if (lines == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "sfdiskM", "lines");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "sfdiskM");
    fprintf (stderr, " \"%s\"", device);
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; lines[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (lines[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "sfdiskM") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.lines.lines_val = (char **) lines;
  for (args.lines.lines_len = 0; lines[args.lines.lines_len]; args.lines.lines_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_SFDISKM,
        (xdrproc_t) xdr_guestfs_sfdiskM_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "sfdiskM", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SFDISKM, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "sfdiskM", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_zfile (guestfs_h *g,
		const char *meth,
		const char *path)
{
  struct guestfs_zfile_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_zfile_ret ret;
  int serial;
  int r;

  if (meth == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zfile", "meth");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zfile", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zfile");
    fprintf (stderr, " \"%s\"", meth);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zfile") == -1) return NULL;
  guestfs___set_busy (g);

  args.meth = (char *) meth;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_ZFILE,
        (xdrproc_t) xdr_guestfs_zfile_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "zfile", &hdr, &err,
        (xdrproc_t) xdr_guestfs_zfile_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZFILE, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zfile", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.description; /* caller will free */
}

struct guestfs_xattr_list *guestfs_getxattrs (guestfs_h *g,
		const char *path)
{
  struct guestfs_getxattrs_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_getxattrs_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "getxattrs", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "getxattrs");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "getxattrs") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_GETXATTRS,
        (xdrproc_t) xdr_guestfs_getxattrs_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "getxattrs", &hdr, &err,
        (xdrproc_t) xdr_guestfs_getxattrs_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GETXATTRS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "getxattrs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.xattrs, sizeof (ret.xattrs));
}

struct guestfs_xattr_list *guestfs_lgetxattrs (guestfs_h *g,
		const char *path)
{
  struct guestfs_lgetxattrs_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_lgetxattrs_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lgetxattrs", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lgetxattrs");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lgetxattrs") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_LGETXATTRS,
        (xdrproc_t) xdr_guestfs_lgetxattrs_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "lgetxattrs", &hdr, &err,
        (xdrproc_t) xdr_guestfs_lgetxattrs_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LGETXATTRS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lgetxattrs", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.xattrs, sizeof (ret.xattrs));
}

int guestfs_setxattr (guestfs_h *g,
		const char *xattr,
		const char *val,
		int vallen,
		const char *path)
{
  struct guestfs_setxattr_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (xattr == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "setxattr", "xattr");
    return -1;
  }
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "setxattr", "val");
    return -1;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "setxattr", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "setxattr");
    fprintf (stderr, " \"%s\"", xattr);
    fprintf (stderr, " \"%s\"", val);
    fprintf (stderr, " %d", vallen);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "setxattr") == -1) return -1;
  guestfs___set_busy (g);

  args.xattr = (char *) xattr;
  args.val = (char *) val;
  args.vallen = vallen;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_SETXATTR,
        (xdrproc_t) xdr_guestfs_setxattr_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "setxattr", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SETXATTR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "setxattr", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_lsetxattr (guestfs_h *g,
		const char *xattr,
		const char *val,
		int vallen,
		const char *path)
{
  struct guestfs_lsetxattr_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (xattr == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lsetxattr", "xattr");
    return -1;
  }
  if (val == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lsetxattr", "val");
    return -1;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lsetxattr", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lsetxattr");
    fprintf (stderr, " \"%s\"", xattr);
    fprintf (stderr, " \"%s\"", val);
    fprintf (stderr, " %d", vallen);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lsetxattr") == -1) return -1;
  guestfs___set_busy (g);

  args.xattr = (char *) xattr;
  args.val = (char *) val;
  args.vallen = vallen;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_LSETXATTR,
        (xdrproc_t) xdr_guestfs_lsetxattr_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lsetxattr", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LSETXATTR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lsetxattr", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_removexattr (guestfs_h *g,
		const char *xattr,
		const char *path)
{
  struct guestfs_removexattr_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (xattr == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "removexattr", "xattr");
    return -1;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "removexattr", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "removexattr");
    fprintf (stderr, " \"%s\"", xattr);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "removexattr") == -1) return -1;
  guestfs___set_busy (g);

  args.xattr = (char *) xattr;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_REMOVEXATTR,
        (xdrproc_t) xdr_guestfs_removexattr_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "removexattr", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_REMOVEXATTR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "removexattr", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_lremovexattr (guestfs_h *g,
		const char *xattr,
		const char *path)
{
  struct guestfs_lremovexattr_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (xattr == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lremovexattr", "xattr");
    return -1;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lremovexattr", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lremovexattr");
    fprintf (stderr, " \"%s\"", xattr);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lremovexattr") == -1) return -1;
  guestfs___set_busy (g);

  args.xattr = (char *) xattr;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_LREMOVEXATTR,
        (xdrproc_t) xdr_guestfs_lremovexattr_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lremovexattr", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LREMOVEXATTR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lremovexattr", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char **guestfs_mountpoints (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_mountpoints_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mountpoints");
    fputc ('\n', stderr);
  }

  if (check_state (g, "mountpoints") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_MOUNTPOINTS, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "mountpoints", &hdr, &err,
        (xdrproc_t) xdr_guestfs_mountpoints_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MOUNTPOINTS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mountpoints", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.mps.mps_val =
    safe_realloc (g, ret.mps.mps_val,
                  sizeof (char *) * (ret.mps.mps_len + 1));
  ret.mps.mps_val[ret.mps.mps_len] = NULL;
  return ret.mps.mps_val;
}

int guestfs_mkmountpoint (guestfs_h *g,
		const char *exemptpath)
{
  struct guestfs_mkmountpoint_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (exemptpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkmountpoint", "exemptpath");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkmountpoint");
    fprintf (stderr, " \"%s\"", exemptpath);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkmountpoint") == -1) return -1;
  guestfs___set_busy (g);

  args.exemptpath = (char *) exemptpath;
  serial = guestfs___send (g, GUESTFS_PROC_MKMOUNTPOINT,
        (xdrproc_t) xdr_guestfs_mkmountpoint_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkmountpoint", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKMOUNTPOINT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkmountpoint", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_rmmountpoint (guestfs_h *g,
		const char *exemptpath)
{
  struct guestfs_rmmountpoint_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (exemptpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "rmmountpoint", "exemptpath");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "rmmountpoint");
    fprintf (stderr, " \"%s\"", exemptpath);
    fputc ('\n', stderr);
  }

  if (check_state (g, "rmmountpoint") == -1) return -1;
  guestfs___set_busy (g);

  args.exemptpath = (char *) exemptpath;
  serial = guestfs___send (g, GUESTFS_PROC_RMMOUNTPOINT,
        (xdrproc_t) xdr_guestfs_rmmountpoint_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "rmmountpoint", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_RMMOUNTPOINT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "rmmountpoint", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_read_file (guestfs_h *g,
		const char *path,
		size_t *size_r)
{
  struct guestfs_read_file_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_read_file_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "read_file", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "read_file");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "read_file") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_READ_FILE,
        (xdrproc_t) xdr_guestfs_read_file_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "read_file", &hdr, &err,
        (xdrproc_t) xdr_guestfs_read_file_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_READ_FILE, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "read_file", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* RBufferOut is tricky: If the buffer is zero-length, then
   * _val might be NULL here.  To make the API saner for
   * callers, we turn this case into a unique pointer (using
   * malloc(1)).
   */
  if (ret.content.content_len > 0) {
    *size_r = ret.content.content_len;
    return ret.content.content_val; /* caller will free */
  } else {
    free (ret.content.content_val);
    char *p = safe_malloc (g, 1);
    *size_r = ret.content.content_len;
    return p;
  }
}

char **guestfs_grep (guestfs_h *g,
		const char *regex,
		const char *path)
{
  struct guestfs_grep_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_grep_ret ret;
  int serial;
  int r;

  if (regex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "grep", "regex");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "grep", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "grep");
    fprintf (stderr, " \"%s\"", regex);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "grep") == -1) return NULL;
  guestfs___set_busy (g);

  args.regex = (char *) regex;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_GREP,
        (xdrproc_t) xdr_guestfs_grep_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "grep", &hdr, &err,
        (xdrproc_t) xdr_guestfs_grep_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GREP, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "grep", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_egrep (guestfs_h *g,
		const char *regex,
		const char *path)
{
  struct guestfs_egrep_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_egrep_ret ret;
  int serial;
  int r;

  if (regex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "egrep", "regex");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "egrep", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "egrep");
    fprintf (stderr, " \"%s\"", regex);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "egrep") == -1) return NULL;
  guestfs___set_busy (g);

  args.regex = (char *) regex;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_EGREP,
        (xdrproc_t) xdr_guestfs_egrep_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "egrep", &hdr, &err,
        (xdrproc_t) xdr_guestfs_egrep_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_EGREP, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "egrep", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_fgrep (guestfs_h *g,
		const char *pattern,
		const char *path)
{
  struct guestfs_fgrep_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_fgrep_ret ret;
  int serial;
  int r;

  if (pattern == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fgrep", "pattern");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fgrep", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "fgrep");
    fprintf (stderr, " \"%s\"", pattern);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "fgrep") == -1) return NULL;
  guestfs___set_busy (g);

  args.pattern = (char *) pattern;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_FGREP,
        (xdrproc_t) xdr_guestfs_fgrep_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "fgrep", &hdr, &err,
        (xdrproc_t) xdr_guestfs_fgrep_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FGREP, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "fgrep", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_grepi (guestfs_h *g,
		const char *regex,
		const char *path)
{
  struct guestfs_grepi_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_grepi_ret ret;
  int serial;
  int r;

  if (regex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "grepi", "regex");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "grepi", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "grepi");
    fprintf (stderr, " \"%s\"", regex);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "grepi") == -1) return NULL;
  guestfs___set_busy (g);

  args.regex = (char *) regex;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_GREPI,
        (xdrproc_t) xdr_guestfs_grepi_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "grepi", &hdr, &err,
        (xdrproc_t) xdr_guestfs_grepi_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GREPI, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "grepi", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_egrepi (guestfs_h *g,
		const char *regex,
		const char *path)
{
  struct guestfs_egrepi_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_egrepi_ret ret;
  int serial;
  int r;

  if (regex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "egrepi", "regex");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "egrepi", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "egrepi");
    fprintf (stderr, " \"%s\"", regex);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "egrepi") == -1) return NULL;
  guestfs___set_busy (g);

  args.regex = (char *) regex;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_EGREPI,
        (xdrproc_t) xdr_guestfs_egrepi_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "egrepi", &hdr, &err,
        (xdrproc_t) xdr_guestfs_egrepi_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_EGREPI, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "egrepi", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_fgrepi (guestfs_h *g,
		const char *pattern,
		const char *path)
{
  struct guestfs_fgrepi_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_fgrepi_ret ret;
  int serial;
  int r;

  if (pattern == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fgrepi", "pattern");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fgrepi", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "fgrepi");
    fprintf (stderr, " \"%s\"", pattern);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "fgrepi") == -1) return NULL;
  guestfs___set_busy (g);

  args.pattern = (char *) pattern;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_FGREPI,
        (xdrproc_t) xdr_guestfs_fgrepi_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "fgrepi", &hdr, &err,
        (xdrproc_t) xdr_guestfs_fgrepi_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FGREPI, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "fgrepi", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_zgrep (guestfs_h *g,
		const char *regex,
		const char *path)
{
  struct guestfs_zgrep_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_zgrep_ret ret;
  int serial;
  int r;

  if (regex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zgrep", "regex");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zgrep", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zgrep");
    fprintf (stderr, " \"%s\"", regex);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zgrep") == -1) return NULL;
  guestfs___set_busy (g);

  args.regex = (char *) regex;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_ZGREP,
        (xdrproc_t) xdr_guestfs_zgrep_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "zgrep", &hdr, &err,
        (xdrproc_t) xdr_guestfs_zgrep_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZGREP, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zgrep", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_zegrep (guestfs_h *g,
		const char *regex,
		const char *path)
{
  struct guestfs_zegrep_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_zegrep_ret ret;
  int serial;
  int r;

  if (regex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zegrep", "regex");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zegrep", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zegrep");
    fprintf (stderr, " \"%s\"", regex);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zegrep") == -1) return NULL;
  guestfs___set_busy (g);

  args.regex = (char *) regex;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_ZEGREP,
        (xdrproc_t) xdr_guestfs_zegrep_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "zegrep", &hdr, &err,
        (xdrproc_t) xdr_guestfs_zegrep_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZEGREP, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zegrep", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_zfgrep (guestfs_h *g,
		const char *pattern,
		const char *path)
{
  struct guestfs_zfgrep_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_zfgrep_ret ret;
  int serial;
  int r;

  if (pattern == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zfgrep", "pattern");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zfgrep", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zfgrep");
    fprintf (stderr, " \"%s\"", pattern);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zfgrep") == -1) return NULL;
  guestfs___set_busy (g);

  args.pattern = (char *) pattern;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_ZFGREP,
        (xdrproc_t) xdr_guestfs_zfgrep_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "zfgrep", &hdr, &err,
        (xdrproc_t) xdr_guestfs_zfgrep_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZFGREP, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zfgrep", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_zgrepi (guestfs_h *g,
		const char *regex,
		const char *path)
{
  struct guestfs_zgrepi_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_zgrepi_ret ret;
  int serial;
  int r;

  if (regex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zgrepi", "regex");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zgrepi", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zgrepi");
    fprintf (stderr, " \"%s\"", regex);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zgrepi") == -1) return NULL;
  guestfs___set_busy (g);

  args.regex = (char *) regex;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_ZGREPI,
        (xdrproc_t) xdr_guestfs_zgrepi_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "zgrepi", &hdr, &err,
        (xdrproc_t) xdr_guestfs_zgrepi_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZGREPI, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zgrepi", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_zegrepi (guestfs_h *g,
		const char *regex,
		const char *path)
{
  struct guestfs_zegrepi_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_zegrepi_ret ret;
  int serial;
  int r;

  if (regex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zegrepi", "regex");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zegrepi", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zegrepi");
    fprintf (stderr, " \"%s\"", regex);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zegrepi") == -1) return NULL;
  guestfs___set_busy (g);

  args.regex = (char *) regex;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_ZEGREPI,
        (xdrproc_t) xdr_guestfs_zegrepi_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "zegrepi", &hdr, &err,
        (xdrproc_t) xdr_guestfs_zegrepi_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZEGREPI, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zegrepi", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char **guestfs_zfgrepi (guestfs_h *g,
		const char *pattern,
		const char *path)
{
  struct guestfs_zfgrepi_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_zfgrepi_ret ret;
  int serial;
  int r;

  if (pattern == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zfgrepi", "pattern");
    return NULL;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zfgrepi", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zfgrepi");
    fprintf (stderr, " \"%s\"", pattern);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zfgrepi") == -1) return NULL;
  guestfs___set_busy (g);

  args.pattern = (char *) pattern;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_ZFGREPI,
        (xdrproc_t) xdr_guestfs_zfgrepi_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "zfgrepi", &hdr, &err,
        (xdrproc_t) xdr_guestfs_zfgrepi_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZFGREPI, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zfgrepi", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.lines.lines_val =
    safe_realloc (g, ret.lines.lines_val,
                  sizeof (char *) * (ret.lines.lines_len + 1));
  ret.lines.lines_val[ret.lines.lines_len] = NULL;
  return ret.lines.lines_val;
}

char *guestfs_realpath (guestfs_h *g,
		const char *path)
{
  struct guestfs_realpath_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_realpath_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "realpath", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "realpath");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "realpath") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_REALPATH,
        (xdrproc_t) xdr_guestfs_realpath_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "realpath", &hdr, &err,
        (xdrproc_t) xdr_guestfs_realpath_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_REALPATH, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "realpath", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.rpath; /* caller will free */
}

int guestfs_ln (guestfs_h *g,
		const char *target,
		const char *linkname)
{
  struct guestfs_ln_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (target == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ln", "target");
    return -1;
  }
  if (linkname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ln", "linkname");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ln");
    fprintf (stderr, " \"%s\"", target);
    fprintf (stderr, " \"%s\"", linkname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ln") == -1) return -1;
  guestfs___set_busy (g);

  args.target = (char *) target;
  args.linkname = (char *) linkname;
  serial = guestfs___send (g, GUESTFS_PROC_LN,
        (xdrproc_t) xdr_guestfs_ln_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "ln", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ln", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_ln_f (guestfs_h *g,
		const char *target,
		const char *linkname)
{
  struct guestfs_ln_f_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (target == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ln_f", "target");
    return -1;
  }
  if (linkname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ln_f", "linkname");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ln_f");
    fprintf (stderr, " \"%s\"", target);
    fprintf (stderr, " \"%s\"", linkname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ln_f") == -1) return -1;
  guestfs___set_busy (g);

  args.target = (char *) target;
  args.linkname = (char *) linkname;
  serial = guestfs___send (g, GUESTFS_PROC_LN_F,
        (xdrproc_t) xdr_guestfs_ln_f_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "ln_f", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LN_F, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ln_f", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_ln_s (guestfs_h *g,
		const char *target,
		const char *linkname)
{
  struct guestfs_ln_s_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (target == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ln_s", "target");
    return -1;
  }
  if (linkname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ln_s", "linkname");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ln_s");
    fprintf (stderr, " \"%s\"", target);
    fprintf (stderr, " \"%s\"", linkname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ln_s") == -1) return -1;
  guestfs___set_busy (g);

  args.target = (char *) target;
  args.linkname = (char *) linkname;
  serial = guestfs___send (g, GUESTFS_PROC_LN_S,
        (xdrproc_t) xdr_guestfs_ln_s_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "ln_s", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LN_S, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ln_s", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_ln_sf (guestfs_h *g,
		const char *target,
		const char *linkname)
{
  struct guestfs_ln_sf_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (target == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ln_sf", "target");
    return -1;
  }
  if (linkname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ln_sf", "linkname");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ln_sf");
    fprintf (stderr, " \"%s\"", target);
    fprintf (stderr, " \"%s\"", linkname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ln_sf") == -1) return -1;
  guestfs___set_busy (g);

  args.target = (char *) target;
  args.linkname = (char *) linkname;
  serial = guestfs___send (g, GUESTFS_PROC_LN_SF,
        (xdrproc_t) xdr_guestfs_ln_sf_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "ln_sf", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LN_SF, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ln_sf", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_readlink (guestfs_h *g,
		const char *path)
{
  struct guestfs_readlink_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_readlink_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "readlink", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "readlink");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "readlink") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_READLINK,
        (xdrproc_t) xdr_guestfs_readlink_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "readlink", &hdr, &err,
        (xdrproc_t) xdr_guestfs_readlink_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_READLINK, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "readlink", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.link; /* caller will free */
}

int guestfs_fallocate (guestfs_h *g,
		const char *path,
		int len)
{
  struct guestfs_fallocate_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fallocate", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "fallocate");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " %d", len);
    fputc ('\n', stderr);
  }

  if (check_state (g, "fallocate") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.len = len;
  serial = guestfs___send (g, GUESTFS_PROC_FALLOCATE,
        (xdrproc_t) xdr_guestfs_fallocate_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "fallocate", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FALLOCATE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "fallocate", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_swapon_device (guestfs_h *g,
		const char *device)
{
  struct guestfs_swapon_device_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "swapon_device", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "swapon_device");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "swapon_device") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_SWAPON_DEVICE,
        (xdrproc_t) xdr_guestfs_swapon_device_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "swapon_device", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SWAPON_DEVICE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "swapon_device", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_swapoff_device (guestfs_h *g,
		const char *device)
{
  struct guestfs_swapoff_device_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "swapoff_device", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "swapoff_device");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "swapoff_device") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_SWAPOFF_DEVICE,
        (xdrproc_t) xdr_guestfs_swapoff_device_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "swapoff_device", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SWAPOFF_DEVICE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "swapoff_device", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_swapon_file (guestfs_h *g,
		const char *file)
{
  struct guestfs_swapon_file_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (file == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "swapon_file", "file");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "swapon_file");
    fprintf (stderr, " \"%s\"", file);
    fputc ('\n', stderr);
  }

  if (check_state (g, "swapon_file") == -1) return -1;
  guestfs___set_busy (g);

  args.file = (char *) file;
  serial = guestfs___send (g, GUESTFS_PROC_SWAPON_FILE,
        (xdrproc_t) xdr_guestfs_swapon_file_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "swapon_file", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SWAPON_FILE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "swapon_file", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_swapoff_file (guestfs_h *g,
		const char *file)
{
  struct guestfs_swapoff_file_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (file == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "swapoff_file", "file");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "swapoff_file");
    fprintf (stderr, " \"%s\"", file);
    fputc ('\n', stderr);
  }

  if (check_state (g, "swapoff_file") == -1) return -1;
  guestfs___set_busy (g);

  args.file = (char *) file;
  serial = guestfs___send (g, GUESTFS_PROC_SWAPOFF_FILE,
        (xdrproc_t) xdr_guestfs_swapoff_file_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "swapoff_file", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SWAPOFF_FILE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "swapoff_file", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_swapon_label (guestfs_h *g,
		const char *label)
{
  struct guestfs_swapon_label_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (label == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "swapon_label", "label");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "swapon_label");
    fprintf (stderr, " \"%s\"", label);
    fputc ('\n', stderr);
  }

  if (check_state (g, "swapon_label") == -1) return -1;
  guestfs___set_busy (g);

  args.label = (char *) label;
  serial = guestfs___send (g, GUESTFS_PROC_SWAPON_LABEL,
        (xdrproc_t) xdr_guestfs_swapon_label_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "swapon_label", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SWAPON_LABEL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "swapon_label", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_swapoff_label (guestfs_h *g,
		const char *label)
{
  struct guestfs_swapoff_label_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (label == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "swapoff_label", "label");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "swapoff_label");
    fprintf (stderr, " \"%s\"", label);
    fputc ('\n', stderr);
  }

  if (check_state (g, "swapoff_label") == -1) return -1;
  guestfs___set_busy (g);

  args.label = (char *) label;
  serial = guestfs___send (g, GUESTFS_PROC_SWAPOFF_LABEL,
        (xdrproc_t) xdr_guestfs_swapoff_label_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "swapoff_label", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SWAPOFF_LABEL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "swapoff_label", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_swapon_uuid (guestfs_h *g,
		const char *uuid)
{
  struct guestfs_swapon_uuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (uuid == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "swapon_uuid", "uuid");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "swapon_uuid");
    fprintf (stderr, " \"%s\"", uuid);
    fputc ('\n', stderr);
  }

  if (check_state (g, "swapon_uuid") == -1) return -1;
  guestfs___set_busy (g);

  args.uuid = (char *) uuid;
  serial = guestfs___send (g, GUESTFS_PROC_SWAPON_UUID,
        (xdrproc_t) xdr_guestfs_swapon_uuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "swapon_uuid", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SWAPON_UUID, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "swapon_uuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_swapoff_uuid (guestfs_h *g,
		const char *uuid)
{
  struct guestfs_swapoff_uuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (uuid == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "swapoff_uuid", "uuid");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "swapoff_uuid");
    fprintf (stderr, " \"%s\"", uuid);
    fputc ('\n', stderr);
  }

  if (check_state (g, "swapoff_uuid") == -1) return -1;
  guestfs___set_busy (g);

  args.uuid = (char *) uuid;
  serial = guestfs___send (g, GUESTFS_PROC_SWAPOFF_UUID,
        (xdrproc_t) xdr_guestfs_swapoff_uuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "swapoff_uuid", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SWAPOFF_UUID, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "swapoff_uuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkswap_file (guestfs_h *g,
		const char *path)
{
  struct guestfs_mkswap_file_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkswap_file", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkswap_file");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkswap_file") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_MKSWAP_FILE,
        (xdrproc_t) xdr_guestfs_mkswap_file_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkswap_file", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKSWAP_FILE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkswap_file", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_inotify_init (guestfs_h *g,
		int maxevents)
{
  struct guestfs_inotify_init_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inotify_init");
    fprintf (stderr, " %d", maxevents);
    fputc ('\n', stderr);
  }

  if (check_state (g, "inotify_init") == -1) return -1;
  guestfs___set_busy (g);

  args.maxevents = maxevents;
  serial = guestfs___send (g, GUESTFS_PROC_INOTIFY_INIT,
        (xdrproc_t) xdr_guestfs_inotify_init_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "inotify_init", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_INOTIFY_INIT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "inotify_init", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int64_t guestfs_inotify_add_watch (guestfs_h *g,
		const char *path,
		int mask)
{
  struct guestfs_inotify_add_watch_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_inotify_add_watch_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "inotify_add_watch", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inotify_add_watch");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " %d", mask);
    fputc ('\n', stderr);
  }

  if (check_state (g, "inotify_add_watch") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.mask = mask;
  serial = guestfs___send (g, GUESTFS_PROC_INOTIFY_ADD_WATCH,
        (xdrproc_t) xdr_guestfs_inotify_add_watch_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "inotify_add_watch", &hdr, &err,
        (xdrproc_t) xdr_guestfs_inotify_add_watch_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_INOTIFY_ADD_WATCH, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "inotify_add_watch", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.wd;
}

int guestfs_inotify_rm_watch (guestfs_h *g,
		int wd)
{
  struct guestfs_inotify_rm_watch_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inotify_rm_watch");
    fprintf (stderr, " %d", wd);
    fputc ('\n', stderr);
  }

  if (check_state (g, "inotify_rm_watch") == -1) return -1;
  guestfs___set_busy (g);

  args.wd = wd;
  serial = guestfs___send (g, GUESTFS_PROC_INOTIFY_RM_WATCH,
        (xdrproc_t) xdr_guestfs_inotify_rm_watch_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "inotify_rm_watch", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_INOTIFY_RM_WATCH, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "inotify_rm_watch", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

struct guestfs_inotify_event_list *guestfs_inotify_read (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_inotify_read_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inotify_read");
    fputc ('\n', stderr);
  }

  if (check_state (g, "inotify_read") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_INOTIFY_READ, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "inotify_read", &hdr, &err,
        (xdrproc_t) xdr_guestfs_inotify_read_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_INOTIFY_READ, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "inotify_read", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.events, sizeof (ret.events));
}

char **guestfs_inotify_files (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_inotify_files_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inotify_files");
    fputc ('\n', stderr);
  }

  if (check_state (g, "inotify_files") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_INOTIFY_FILES, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "inotify_files", &hdr, &err,
        (xdrproc_t) xdr_guestfs_inotify_files_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_INOTIFY_FILES, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "inotify_files", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.paths.paths_val =
    safe_realloc (g, ret.paths.paths_val,
                  sizeof (char *) * (ret.paths.paths_len + 1));
  ret.paths.paths_val[ret.paths.paths_len] = NULL;
  return ret.paths.paths_val;
}

int guestfs_inotify_close (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "inotify_close");
    fputc ('\n', stderr);
  }

  if (check_state (g, "inotify_close") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_INOTIFY_CLOSE, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "inotify_close", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_INOTIFY_CLOSE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "inotify_close", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_setcon (guestfs_h *g,
		const char *context)
{
  struct guestfs_setcon_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (context == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "setcon", "context");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "setcon");
    fprintf (stderr, " \"%s\"", context);
    fputc ('\n', stderr);
  }

  if (check_state (g, "setcon") == -1) return -1;
  guestfs___set_busy (g);

  args.context = (char *) context;
  serial = guestfs___send (g, GUESTFS_PROC_SETCON,
        (xdrproc_t) xdr_guestfs_setcon_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "setcon", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_SETCON, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "setcon", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_getcon (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_getcon_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "getcon");
    fputc ('\n', stderr);
  }

  if (check_state (g, "getcon") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_GETCON, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "getcon", &hdr, &err,
        (xdrproc_t) xdr_guestfs_getcon_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GETCON, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "getcon", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.context; /* caller will free */
}

int guestfs_mkfs_b (guestfs_h *g,
		const char *fstype,
		int blocksize,
		const char *device)
{
  struct guestfs_mkfs_b_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (fstype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkfs_b", "fstype");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkfs_b", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkfs_b");
    fprintf (stderr, " \"%s\"", fstype);
    fprintf (stderr, " %d", blocksize);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkfs_b") == -1) return -1;
  guestfs___set_busy (g);

  args.fstype = (char *) fstype;
  args.blocksize = blocksize;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_MKFS_B,
        (xdrproc_t) xdr_guestfs_mkfs_b_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkfs_b", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKFS_B, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkfs_b", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mke2journal (guestfs_h *g,
		int blocksize,
		const char *device)
{
  struct guestfs_mke2journal_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2journal", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mke2journal");
    fprintf (stderr, " %d", blocksize);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mke2journal") == -1) return -1;
  guestfs___set_busy (g);

  args.blocksize = blocksize;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_MKE2JOURNAL,
        (xdrproc_t) xdr_guestfs_mke2journal_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mke2journal", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKE2JOURNAL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mke2journal", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mke2journal_L (guestfs_h *g,
		int blocksize,
		const char *label,
		const char *device)
{
  struct guestfs_mke2journal_L_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (label == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2journal_L", "label");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2journal_L", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mke2journal_L");
    fprintf (stderr, " %d", blocksize);
    fprintf (stderr, " \"%s\"", label);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mke2journal_L") == -1) return -1;
  guestfs___set_busy (g);

  args.blocksize = blocksize;
  args.label = (char *) label;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_MKE2JOURNAL_L,
        (xdrproc_t) xdr_guestfs_mke2journal_L_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mke2journal_L", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKE2JOURNAL_L, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mke2journal_L", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mke2journal_U (guestfs_h *g,
		int blocksize,
		const char *uuid,
		const char *device)
{
  struct guestfs_mke2journal_U_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (uuid == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2journal_U", "uuid");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2journal_U", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mke2journal_U");
    fprintf (stderr, " %d", blocksize);
    fprintf (stderr, " \"%s\"", uuid);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mke2journal_U") == -1) return -1;
  guestfs___set_busy (g);

  args.blocksize = blocksize;
  args.uuid = (char *) uuid;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_MKE2JOURNAL_U,
        (xdrproc_t) xdr_guestfs_mke2journal_U_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mke2journal_U", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKE2JOURNAL_U, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mke2journal_U", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mke2fs_J (guestfs_h *g,
		const char *fstype,
		int blocksize,
		const char *device,
		const char *journal)
{
  struct guestfs_mke2fs_J_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (fstype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_J", "fstype");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_J", "device");
    return -1;
  }
  if (journal == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_J", "journal");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mke2fs_J");
    fprintf (stderr, " \"%s\"", fstype);
    fprintf (stderr, " %d", blocksize);
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", journal);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mke2fs_J") == -1) return -1;
  guestfs___set_busy (g);

  args.fstype = (char *) fstype;
  args.blocksize = blocksize;
  args.device = (char *) device;
  args.journal = (char *) journal;
  serial = guestfs___send (g, GUESTFS_PROC_MKE2FS_J,
        (xdrproc_t) xdr_guestfs_mke2fs_J_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mke2fs_J", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKE2FS_J, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mke2fs_J", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mke2fs_JL (guestfs_h *g,
		const char *fstype,
		int blocksize,
		const char *device,
		const char *label)
{
  struct guestfs_mke2fs_JL_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (fstype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_JL", "fstype");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_JL", "device");
    return -1;
  }
  if (label == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_JL", "label");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mke2fs_JL");
    fprintf (stderr, " \"%s\"", fstype);
    fprintf (stderr, " %d", blocksize);
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", label);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mke2fs_JL") == -1) return -1;
  guestfs___set_busy (g);

  args.fstype = (char *) fstype;
  args.blocksize = blocksize;
  args.device = (char *) device;
  args.label = (char *) label;
  serial = guestfs___send (g, GUESTFS_PROC_MKE2FS_JL,
        (xdrproc_t) xdr_guestfs_mke2fs_JL_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mke2fs_JL", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKE2FS_JL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mke2fs_JL", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mke2fs_JU (guestfs_h *g,
		const char *fstype,
		int blocksize,
		const char *device,
		const char *uuid)
{
  struct guestfs_mke2fs_JU_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (fstype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_JU", "fstype");
    return -1;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_JU", "device");
    return -1;
  }
  if (uuid == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mke2fs_JU", "uuid");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mke2fs_JU");
    fprintf (stderr, " \"%s\"", fstype);
    fprintf (stderr, " %d", blocksize);
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", uuid);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mke2fs_JU") == -1) return -1;
  guestfs___set_busy (g);

  args.fstype = (char *) fstype;
  args.blocksize = blocksize;
  args.device = (char *) device;
  args.uuid = (char *) uuid;
  serial = guestfs___send (g, GUESTFS_PROC_MKE2FS_JU,
        (xdrproc_t) xdr_guestfs_mke2fs_JU_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mke2fs_JU", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKE2FS_JU, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mke2fs_JU", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_modprobe (guestfs_h *g,
		const char *modulename)
{
  struct guestfs_modprobe_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (modulename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "modprobe", "modulename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "modprobe");
    fprintf (stderr, " \"%s\"", modulename);
    fputc ('\n', stderr);
  }

  if (check_state (g, "modprobe") == -1) return -1;
  guestfs___set_busy (g);

  args.modulename = (char *) modulename;
  serial = guestfs___send (g, GUESTFS_PROC_MODPROBE,
        (xdrproc_t) xdr_guestfs_modprobe_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "modprobe", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MODPROBE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "modprobe", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_echo_daemon (guestfs_h *g,
		char *const *words)
{
  struct guestfs_echo_daemon_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_echo_daemon_ret ret;
  int serial;
  int r;

  if (words == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "echo_daemon", "words");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "echo_daemon");
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; words[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (words[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "echo_daemon") == -1) return NULL;
  guestfs___set_busy (g);

  args.words.words_val = (char **) words;
  for (args.words.words_len = 0; words[args.words.words_len]; args.words.words_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_ECHO_DAEMON,
        (xdrproc_t) xdr_guestfs_echo_daemon_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "echo_daemon", &hdr, &err,
        (xdrproc_t) xdr_guestfs_echo_daemon_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ECHO_DAEMON, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "echo_daemon", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.output; /* caller will free */
}

int guestfs_find0 (guestfs_h *g,
		const char *directory,
		const char *files)
{
  struct guestfs_find0_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "find0", "directory");
    return -1;
  }
  if (files == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "find0", "files");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "find0");
    fprintf (stderr, " \"%s\"", directory);
    fprintf (stderr, " \"%s\"", files);
    fputc ('\n', stderr);
  }

  if (check_state (g, "find0") == -1) return -1;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_FIND0,
        (xdrproc_t) xdr_guestfs_find0_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "find0", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FIND0, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "find0", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  if (guestfs___recv_file (g, files) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_case_sensitive_path (guestfs_h *g,
		const char *path)
{
  struct guestfs_case_sensitive_path_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_case_sensitive_path_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "case_sensitive_path", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "case_sensitive_path");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "case_sensitive_path") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_CASE_SENSITIVE_PATH,
        (xdrproc_t) xdr_guestfs_case_sensitive_path_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "case_sensitive_path", &hdr, &err,
        (xdrproc_t) xdr_guestfs_case_sensitive_path_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CASE_SENSITIVE_PATH, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "case_sensitive_path", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.rpath; /* caller will free */
}

char *guestfs_vfs_type (guestfs_h *g,
		const char *device)
{
  struct guestfs_vfs_type_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_vfs_type_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vfs_type", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vfs_type");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vfs_type") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_VFS_TYPE,
        (xdrproc_t) xdr_guestfs_vfs_type_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "vfs_type", &hdr, &err,
        (xdrproc_t) xdr_guestfs_vfs_type_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VFS_TYPE, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vfs_type", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.fstype; /* caller will free */
}

int guestfs_truncate (guestfs_h *g,
		const char *path)
{
  struct guestfs_truncate_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "truncate", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "truncate");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "truncate") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_TRUNCATE,
        (xdrproc_t) xdr_guestfs_truncate_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "truncate", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TRUNCATE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "truncate", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_truncate_size (guestfs_h *g,
		const char *path,
		int64_t size)
{
  struct guestfs_truncate_size_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "truncate_size", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "truncate_size");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " %" PRIi64, size);
    fputc ('\n', stderr);
  }

  if (check_state (g, "truncate_size") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.size = size;
  serial = guestfs___send (g, GUESTFS_PROC_TRUNCATE_SIZE,
        (xdrproc_t) xdr_guestfs_truncate_size_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "truncate_size", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TRUNCATE_SIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "truncate_size", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_utimens (guestfs_h *g,
		const char *path,
		int64_t atsecs,
		int64_t atnsecs,
		int64_t mtsecs,
		int64_t mtnsecs)
{
  struct guestfs_utimens_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "utimens", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "utimens");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " %" PRIi64, atsecs);
    fprintf (stderr, " %" PRIi64, atnsecs);
    fprintf (stderr, " %" PRIi64, mtsecs);
    fprintf (stderr, " %" PRIi64, mtnsecs);
    fputc ('\n', stderr);
  }

  if (check_state (g, "utimens") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.atsecs = atsecs;
  args.atnsecs = atnsecs;
  args.mtsecs = mtsecs;
  args.mtnsecs = mtnsecs;
  serial = guestfs___send (g, GUESTFS_PROC_UTIMENS,
        (xdrproc_t) xdr_guestfs_utimens_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "utimens", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_UTIMENS, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "utimens", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_mkdir_mode (guestfs_h *g,
		const char *path,
		int mode)
{
  struct guestfs_mkdir_mode_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "mkdir_mode", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "mkdir_mode");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " %d", mode);
    fputc ('\n', stderr);
  }

  if (check_state (g, "mkdir_mode") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.mode = mode;
  serial = guestfs___send (g, GUESTFS_PROC_MKDIR_MODE,
        (xdrproc_t) xdr_guestfs_mkdir_mode_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "mkdir_mode", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_MKDIR_MODE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "mkdir_mode", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_lchown (guestfs_h *g,
		int owner,
		int group,
		const char *path)
{
  struct guestfs_lchown_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lchown", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lchown");
    fprintf (stderr, " %d", owner);
    fprintf (stderr, " %d", group);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lchown") == -1) return -1;
  guestfs___set_busy (g);

  args.owner = owner;
  args.group = group;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_LCHOWN,
        (xdrproc_t) xdr_guestfs_lchown_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lchown", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LCHOWN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lchown", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

struct guestfs_stat_list *guestfs_lstatlist (guestfs_h *g,
		const char *path,
		char *const *names)
{
  struct guestfs_lstatlist_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_lstatlist_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lstatlist", "path");
    return NULL;
  }
  if (names == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lstatlist", "names");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "lstatlist");
    fprintf (stderr, " \"%s\"", path);
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; names[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (names[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lstatlist") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.names.names_val = (char **) names;
  for (args.names.names_len = 0; names[args.names.names_len]; args.names.names_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_LSTATLIST,
        (xdrproc_t) xdr_guestfs_lstatlist_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "lstatlist", &hdr, &err,
        (xdrproc_t) xdr_guestfs_lstatlist_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LSTATLIST, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lstatlist", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.statbufs, sizeof (ret.statbufs));
}

struct guestfs_xattr_list *guestfs_lxattrlist (guestfs_h *g,
		const char *path,
		char *const *names)
{
  struct guestfs_lxattrlist_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_lxattrlist_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lxattrlist", "path");
    return NULL;
  }
  if (names == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lxattrlist", "names");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "lxattrlist");
    fprintf (stderr, " \"%s\"", path);
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; names[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (names[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lxattrlist") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.names.names_val = (char **) names;
  for (args.names.names_len = 0; names[args.names.names_len]; args.names.names_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_LXATTRLIST,
        (xdrproc_t) xdr_guestfs_lxattrlist_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "lxattrlist", &hdr, &err,
        (xdrproc_t) xdr_guestfs_lxattrlist_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LXATTRLIST, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lxattrlist", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.xattrs, sizeof (ret.xattrs));
}

char **guestfs_readlinklist (guestfs_h *g,
		const char *path,
		char *const *names)
{
  struct guestfs_readlinklist_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_readlinklist_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "readlinklist", "path");
    return NULL;
  }
  if (names == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "readlinklist", "names");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "readlinklist");
    fprintf (stderr, " \"%s\"", path);
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; names[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (names[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "readlinklist") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.names.names_val = (char **) names;
  for (args.names.names_len = 0; names[args.names.names_len]; args.names.names_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_READLINKLIST,
        (xdrproc_t) xdr_guestfs_readlinklist_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "readlinklist", &hdr, &err,
        (xdrproc_t) xdr_guestfs_readlinklist_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_READLINKLIST, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "readlinklist", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.links.links_val =
    safe_realloc (g, ret.links.links_val,
                  sizeof (char *) * (ret.links.links_len + 1));
  ret.links.links_val[ret.links.links_len] = NULL;
  return ret.links.links_val;
}

char *guestfs_pread (guestfs_h *g,
		const char *path,
		int count,
		int64_t offset,
		size_t *size_r)
{
  struct guestfs_pread_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_pread_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pread", "path");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pread");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " %d", count);
    fprintf (stderr, " %" PRIi64, offset);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pread") == -1) return NULL;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.count = count;
  args.offset = offset;
  serial = guestfs___send (g, GUESTFS_PROC_PREAD,
        (xdrproc_t) xdr_guestfs_pread_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "pread", &hdr, &err,
        (xdrproc_t) xdr_guestfs_pread_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PREAD, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pread", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* RBufferOut is tricky: If the buffer is zero-length, then
   * _val might be NULL here.  To make the API saner for
   * callers, we turn this case into a unique pointer (using
   * malloc(1)).
   */
  if (ret.content.content_len > 0) {
    *size_r = ret.content.content_len;
    return ret.content.content_val; /* caller will free */
  } else {
    free (ret.content.content_val);
    char *p = safe_malloc (g, 1);
    *size_r = ret.content.content_len;
    return p;
  }
}

int guestfs_part_init (guestfs_h *g,
		const char *device,
		const char *parttype)
{
  struct guestfs_part_init_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_init", "device");
    return -1;
  }
  if (parttype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_init", "parttype");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_init");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", parttype);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_init") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.parttype = (char *) parttype;
  serial = guestfs___send (g, GUESTFS_PROC_PART_INIT,
        (xdrproc_t) xdr_guestfs_part_init_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "part_init", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_INIT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_init", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_part_add (guestfs_h *g,
		const char *device,
		const char *prlogex,
		int64_t startsect,
		int64_t endsect)
{
  struct guestfs_part_add_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_add", "device");
    return -1;
  }
  if (prlogex == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_add", "prlogex");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_add");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", prlogex);
    fprintf (stderr, " %" PRIi64, startsect);
    fprintf (stderr, " %" PRIi64, endsect);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_add") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.prlogex = (char *) prlogex;
  args.startsect = startsect;
  args.endsect = endsect;
  serial = guestfs___send (g, GUESTFS_PROC_PART_ADD,
        (xdrproc_t) xdr_guestfs_part_add_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "part_add", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_ADD, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_add", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_part_disk (guestfs_h *g,
		const char *device,
		const char *parttype)
{
  struct guestfs_part_disk_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_disk", "device");
    return -1;
  }
  if (parttype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_disk", "parttype");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_disk");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", parttype);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_disk") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.parttype = (char *) parttype;
  serial = guestfs___send (g, GUESTFS_PROC_PART_DISK,
        (xdrproc_t) xdr_guestfs_part_disk_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "part_disk", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_DISK, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_disk", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_part_set_bootable (guestfs_h *g,
		const char *device,
		int partnum,
		int bootable)
{
  struct guestfs_part_set_bootable_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_set_bootable", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_set_bootable");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", partnum);
    fputs (bootable ? " true" : " false", stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_set_bootable") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.partnum = partnum;
  args.bootable = bootable;
  serial = guestfs___send (g, GUESTFS_PROC_PART_SET_BOOTABLE,
        (xdrproc_t) xdr_guestfs_part_set_bootable_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "part_set_bootable", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_SET_BOOTABLE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_set_bootable", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_part_set_name (guestfs_h *g,
		const char *device,
		int partnum,
		const char *name)
{
  struct guestfs_part_set_name_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_set_name", "device");
    return -1;
  }
  if (name == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_set_name", "name");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_set_name");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", partnum);
    fprintf (stderr, " \"%s\"", name);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_set_name") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.partnum = partnum;
  args.name = (char *) name;
  serial = guestfs___send (g, GUESTFS_PROC_PART_SET_NAME,
        (xdrproc_t) xdr_guestfs_part_set_name_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "part_set_name", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_SET_NAME, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_set_name", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

struct guestfs_partition_list *guestfs_part_list (guestfs_h *g,
		const char *device)
{
  struct guestfs_part_list_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_part_list_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_list", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_list");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_list") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_PART_LIST,
        (xdrproc_t) xdr_guestfs_part_list_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "part_list", &hdr, &err,
        (xdrproc_t) xdr_guestfs_part_list_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_LIST, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_list", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this */
  return safe_memdup (g, &ret.partitions, sizeof (ret.partitions));
}

char *guestfs_part_get_parttype (guestfs_h *g,
		const char *device)
{
  struct guestfs_part_get_parttype_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_part_get_parttype_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_get_parttype", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_get_parttype");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_get_parttype") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_PART_GET_PARTTYPE,
        (xdrproc_t) xdr_guestfs_part_get_parttype_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "part_get_parttype", &hdr, &err,
        (xdrproc_t) xdr_guestfs_part_get_parttype_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_GET_PARTTYPE, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_get_parttype", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.parttype; /* caller will free */
}

int guestfs_fill (guestfs_h *g,
		int c,
		int len,
		const char *path)
{
  struct guestfs_fill_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fill", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "fill");
    fprintf (stderr, " %d", c);
    fprintf (stderr, " %d", len);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "fill") == -1) return -1;
  guestfs___set_busy (g);

  args.c = c;
  args.len = len;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_FILL,
        (xdrproc_t) xdr_guestfs_fill_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "fill", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FILL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "fill", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_available (guestfs_h *g,
		char *const *groups)
{
  struct guestfs_available_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (groups == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "available", "groups");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "available");
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; groups[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (groups[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "available") == -1) return -1;
  guestfs___set_busy (g);

  args.groups.groups_val = (char **) groups;
  for (args.groups.groups_len = 0; groups[args.groups.groups_len]; args.groups.groups_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_AVAILABLE,
        (xdrproc_t) xdr_guestfs_available_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "available", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AVAILABLE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "available", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_dd (guestfs_h *g,
		const char *src,
		const char *dest)
{
  struct guestfs_dd_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (src == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "dd", "src");
    return -1;
  }
  if (dest == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "dd", "dest");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "dd");
    fprintf (stderr, " \"%s\"", src);
    fprintf (stderr, " \"%s\"", dest);
    fputc ('\n', stderr);
  }

  if (check_state (g, "dd") == -1) return -1;
  guestfs___set_busy (g);

  args.src = (char *) src;
  args.dest = (char *) dest;
  serial = guestfs___send (g, GUESTFS_PROC_DD,
        (xdrproc_t) xdr_guestfs_dd_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "dd", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DD, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "dd", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int64_t guestfs_filesize (guestfs_h *g,
		const char *file)
{
  struct guestfs_filesize_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_filesize_ret ret;
  int serial;
  int r;

  if (file == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "filesize", "file");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "filesize");
    fprintf (stderr, " \"%s\"", file);
    fputc ('\n', stderr);
  }

  if (check_state (g, "filesize") == -1) return -1;
  guestfs___set_busy (g);

  args.file = (char *) file;
  serial = guestfs___send (g, GUESTFS_PROC_FILESIZE,
        (xdrproc_t) xdr_guestfs_filesize_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "filesize", &hdr, &err,
        (xdrproc_t) xdr_guestfs_filesize_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FILESIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "filesize", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.size;
}

int guestfs_lvrename (guestfs_h *g,
		const char *logvol,
		const char *newlogvol)
{
  struct guestfs_lvrename_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (logvol == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvrename", "logvol");
    return -1;
  }
  if (newlogvol == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvrename", "newlogvol");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvrename");
    fprintf (stderr, " \"%s\"", logvol);
    fprintf (stderr, " \"%s\"", newlogvol);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvrename") == -1) return -1;
  guestfs___set_busy (g);

  args.logvol = (char *) logvol;
  args.newlogvol = (char *) newlogvol;
  serial = guestfs___send (g, GUESTFS_PROC_LVRENAME,
        (xdrproc_t) xdr_guestfs_lvrename_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lvrename", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVRENAME, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvrename", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_vgrename (guestfs_h *g,
		const char *volgroup,
		const char *newvolgroup)
{
  struct guestfs_vgrename_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (volgroup == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vgrename", "volgroup");
    return -1;
  }
  if (newvolgroup == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vgrename", "newvolgroup");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vgrename");
    fprintf (stderr, " \"%s\"", volgroup);
    fprintf (stderr, " \"%s\"", newvolgroup);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vgrename") == -1) return -1;
  guestfs___set_busy (g);

  args.volgroup = (char *) volgroup;
  args.newvolgroup = (char *) newvolgroup;
  serial = guestfs___send (g, GUESTFS_PROC_VGRENAME,
        (xdrproc_t) xdr_guestfs_vgrename_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "vgrename", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGRENAME, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vgrename", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_initrd_cat (guestfs_h *g,
		const char *initrdpath,
		const char *filename,
		size_t *size_r)
{
  struct guestfs_initrd_cat_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_initrd_cat_ret ret;
  int serial;
  int r;

  if (initrdpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "initrd_cat", "initrdpath");
    return NULL;
  }
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "initrd_cat", "filename");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "initrd_cat");
    fprintf (stderr, " \"%s\"", initrdpath);
    fprintf (stderr, " \"%s\"", filename);
    fputc ('\n', stderr);
  }

  if (check_state (g, "initrd_cat") == -1) return NULL;
  guestfs___set_busy (g);

  args.initrdpath = (char *) initrdpath;
  args.filename = (char *) filename;
  serial = guestfs___send (g, GUESTFS_PROC_INITRD_CAT,
        (xdrproc_t) xdr_guestfs_initrd_cat_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "initrd_cat", &hdr, &err,
        (xdrproc_t) xdr_guestfs_initrd_cat_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_INITRD_CAT, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "initrd_cat", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* RBufferOut is tricky: If the buffer is zero-length, then
   * _val might be NULL here.  To make the API saner for
   * callers, we turn this case into a unique pointer (using
   * malloc(1)).
   */
  if (ret.content.content_len > 0) {
    *size_r = ret.content.content_len;
    return ret.content.content_val; /* caller will free */
  } else {
    free (ret.content.content_val);
    char *p = safe_malloc (g, 1);
    *size_r = ret.content.content_len;
    return p;
  }
}

char *guestfs_pvuuid (guestfs_h *g,
		const char *device)
{
  struct guestfs_pvuuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_pvuuid_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pvuuid", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pvuuid");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pvuuid") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_PVUUID,
        (xdrproc_t) xdr_guestfs_pvuuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "pvuuid", &hdr, &err,
        (xdrproc_t) xdr_guestfs_pvuuid_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PVUUID, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pvuuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.uuid; /* caller will free */
}

char *guestfs_vguuid (guestfs_h *g,
		const char *vgname)
{
  struct guestfs_vguuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_vguuid_ret ret;
  int serial;
  int r;

  if (vgname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vguuid", "vgname");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vguuid");
    fprintf (stderr, " \"%s\"", vgname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vguuid") == -1) return NULL;
  guestfs___set_busy (g);

  args.vgname = (char *) vgname;
  serial = guestfs___send (g, GUESTFS_PROC_VGUUID,
        (xdrproc_t) xdr_guestfs_vguuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "vguuid", &hdr, &err,
        (xdrproc_t) xdr_guestfs_vguuid_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGUUID, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vguuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.uuid; /* caller will free */
}

char *guestfs_lvuuid (guestfs_h *g,
		const char *device)
{
  struct guestfs_lvuuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_lvuuid_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvuuid", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvuuid");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvuuid") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_LVUUID,
        (xdrproc_t) xdr_guestfs_lvuuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "lvuuid", &hdr, &err,
        (xdrproc_t) xdr_guestfs_lvuuid_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVUUID, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvuuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.uuid; /* caller will free */
}

char **guestfs_vgpvuuids (guestfs_h *g,
		const char *vgname)
{
  struct guestfs_vgpvuuids_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_vgpvuuids_ret ret;
  int serial;
  int r;

  if (vgname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vgpvuuids", "vgname");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vgpvuuids");
    fprintf (stderr, " \"%s\"", vgname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vgpvuuids") == -1) return NULL;
  guestfs___set_busy (g);

  args.vgname = (char *) vgname;
  serial = guestfs___send (g, GUESTFS_PROC_VGPVUUIDS,
        (xdrproc_t) xdr_guestfs_vgpvuuids_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "vgpvuuids", &hdr, &err,
        (xdrproc_t) xdr_guestfs_vgpvuuids_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGPVUUIDS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vgpvuuids", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.uuids.uuids_val =
    safe_realloc (g, ret.uuids.uuids_val,
                  sizeof (char *) * (ret.uuids.uuids_len + 1));
  ret.uuids.uuids_val[ret.uuids.uuids_len] = NULL;
  return ret.uuids.uuids_val;
}

char **guestfs_vglvuuids (guestfs_h *g,
		const char *vgname)
{
  struct guestfs_vglvuuids_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_vglvuuids_ret ret;
  int serial;
  int r;

  if (vgname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vglvuuids", "vgname");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vglvuuids");
    fprintf (stderr, " \"%s\"", vgname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vglvuuids") == -1) return NULL;
  guestfs___set_busy (g);

  args.vgname = (char *) vgname;
  serial = guestfs___send (g, GUESTFS_PROC_VGLVUUIDS,
        (xdrproc_t) xdr_guestfs_vglvuuids_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "vglvuuids", &hdr, &err,
        (xdrproc_t) xdr_guestfs_vglvuuids_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGLVUUIDS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vglvuuids", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.uuids.uuids_val =
    safe_realloc (g, ret.uuids.uuids_val,
                  sizeof (char *) * (ret.uuids.uuids_len + 1));
  ret.uuids.uuids_val[ret.uuids.uuids_len] = NULL;
  return ret.uuids.uuids_val;
}

int guestfs_copy_size (guestfs_h *g,
		const char *src,
		const char *dest,
		int64_t size)
{
  struct guestfs_copy_size_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (src == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "copy_size", "src");
    return -1;
  }
  if (dest == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "copy_size", "dest");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "copy_size");
    fprintf (stderr, " \"%s\"", src);
    fprintf (stderr, " \"%s\"", dest);
    fprintf (stderr, " %" PRIi64, size);
    fputc ('\n', stderr);
  }

  if (check_state (g, "copy_size") == -1) return -1;
  guestfs___set_busy (g);

  args.src = (char *) src;
  args.dest = (char *) dest;
  args.size = size;
  serial = guestfs___send (g, GUESTFS_PROC_COPY_SIZE,
        (xdrproc_t) xdr_guestfs_copy_size_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "copy_size", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_COPY_SIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "copy_size", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_zero_device (guestfs_h *g,
		const char *device)
{
  struct guestfs_zero_device_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "zero_device", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "zero_device");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "zero_device") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_ZERO_DEVICE,
        (xdrproc_t) xdr_guestfs_zero_device_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "zero_device", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_ZERO_DEVICE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "zero_device", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_txz_in (guestfs_h *g,
		const char *tarball,
		const char *directory)
{
  struct guestfs_txz_in_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (tarball == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "txz_in", "tarball");
    return -1;
  }
  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "txz_in", "directory");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "txz_in");
    fprintf (stderr, " \"%s\"", tarball);
    fprintf (stderr, " \"%s\"", directory);
    fputc ('\n', stderr);
  }

  if (check_state (g, "txz_in") == -1) return -1;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_TXZ_IN,
        (xdrproc_t) xdr_guestfs_txz_in_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  r = guestfs___send_file (g, tarball);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }
  if (r == -2) /* daemon cancelled */
    goto read_reply;

 read_reply:
  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "txz_in", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TXZ_IN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "txz_in", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_txz_out (guestfs_h *g,
		const char *directory,
		const char *tarball)
{
  struct guestfs_txz_out_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "txz_out", "directory");
    return -1;
  }
  if (tarball == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "txz_out", "tarball");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "txz_out");
    fprintf (stderr, " \"%s\"", directory);
    fprintf (stderr, " \"%s\"", tarball);
    fputc ('\n', stderr);
  }

  if (check_state (g, "txz_out") == -1) return -1;
  guestfs___set_busy (g);

  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_TXZ_OUT,
        (xdrproc_t) xdr_guestfs_txz_out_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "txz_out", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_TXZ_OUT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "txz_out", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  if (guestfs___recv_file (g, tarball) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_ntfsresize (guestfs_h *g,
		const char *device)
{
  struct guestfs_ntfsresize_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ntfsresize", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ntfsresize");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ntfsresize") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_NTFSRESIZE,
        (xdrproc_t) xdr_guestfs_ntfsresize_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "ntfsresize", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_NTFSRESIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ntfsresize", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_vgscan (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vgscan");
    fputc ('\n', stderr);
  }

  if (check_state (g, "vgscan") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_VGSCAN, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "vgscan", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VGSCAN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vgscan", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_part_del (guestfs_h *g,
		const char *device,
		int partnum)
{
  struct guestfs_part_del_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_del", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_del");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", partnum);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_del") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.partnum = partnum;
  serial = guestfs___send (g, GUESTFS_PROC_PART_DEL,
        (xdrproc_t) xdr_guestfs_part_del_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "part_del", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_DEL, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_del", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_part_get_bootable (guestfs_h *g,
		const char *device,
		int partnum)
{
  struct guestfs_part_get_bootable_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_part_get_bootable_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_get_bootable", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_get_bootable");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", partnum);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_get_bootable") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.partnum = partnum;
  serial = guestfs___send (g, GUESTFS_PROC_PART_GET_BOOTABLE,
        (xdrproc_t) xdr_guestfs_part_get_bootable_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "part_get_bootable", &hdr, &err,
        (xdrproc_t) xdr_guestfs_part_get_bootable_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_GET_BOOTABLE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_get_bootable", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.bootable;
}

int guestfs_part_get_mbr_id (guestfs_h *g,
		const char *device,
		int partnum)
{
  struct guestfs_part_get_mbr_id_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_part_get_mbr_id_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_get_mbr_id", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_get_mbr_id");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", partnum);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_get_mbr_id") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.partnum = partnum;
  serial = guestfs___send (g, GUESTFS_PROC_PART_GET_MBR_ID,
        (xdrproc_t) xdr_guestfs_part_get_mbr_id_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "part_get_mbr_id", &hdr, &err,
        (xdrproc_t) xdr_guestfs_part_get_mbr_id_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_GET_MBR_ID, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_get_mbr_id", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.idbyte;
}

int guestfs_part_set_mbr_id (guestfs_h *g,
		const char *device,
		int partnum,
		int idbyte)
{
  struct guestfs_part_set_mbr_id_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_set_mbr_id", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_set_mbr_id");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", partnum);
    fprintf (stderr, " %d", idbyte);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_set_mbr_id") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.partnum = partnum;
  args.idbyte = idbyte;
  serial = guestfs___send (g, GUESTFS_PROC_PART_SET_MBR_ID,
        (xdrproc_t) xdr_guestfs_part_set_mbr_id_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "part_set_mbr_id", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_SET_MBR_ID, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_set_mbr_id", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_checksum_device (guestfs_h *g,
		const char *csumtype,
		const char *device)
{
  struct guestfs_checksum_device_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_checksum_device_ret ret;
  int serial;
  int r;

  if (csumtype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "checksum_device", "csumtype");
    return NULL;
  }
  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "checksum_device", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "checksum_device");
    fprintf (stderr, " \"%s\"", csumtype);
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "checksum_device") == -1) return NULL;
  guestfs___set_busy (g);

  args.csumtype = (char *) csumtype;
  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_CHECKSUM_DEVICE,
        (xdrproc_t) xdr_guestfs_checksum_device_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "checksum_device", &hdr, &err,
        (xdrproc_t) xdr_guestfs_checksum_device_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CHECKSUM_DEVICE, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "checksum_device", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.checksum; /* caller will free */
}

int guestfs_lvresize_free (guestfs_h *g,
		const char *lv,
		int percent)
{
  struct guestfs_lvresize_free_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (lv == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvresize_free", "lv");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvresize_free");
    fprintf (stderr, " \"%s\"", lv);
    fprintf (stderr, " %d", percent);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvresize_free") == -1) return -1;
  guestfs___set_busy (g);

  args.lv = (char *) lv;
  args.percent = percent;
  serial = guestfs___send (g, GUESTFS_PROC_LVRESIZE_FREE,
        (xdrproc_t) xdr_guestfs_lvresize_free_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lvresize_free", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVRESIZE_FREE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvresize_free", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_aug_clear (guestfs_h *g,
		const char *augpath)
{
  struct guestfs_aug_clear_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (augpath == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "aug_clear", "augpath");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "aug_clear");
    fprintf (stderr, " \"%s\"", augpath);
    fputc ('\n', stderr);
  }

  if (check_state (g, "aug_clear") == -1) return -1;
  guestfs___set_busy (g);

  args.augpath = (char *) augpath;
  serial = guestfs___send (g, GUESTFS_PROC_AUG_CLEAR,
        (xdrproc_t) xdr_guestfs_aug_clear_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "aug_clear", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AUG_CLEAR, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "aug_clear", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_get_umask (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_get_umask_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "get_umask");
    fputc ('\n', stderr);
  }

  if (check_state (g, "get_umask") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_GET_UMASK, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "get_umask", &hdr, &err,
        (xdrproc_t) xdr_guestfs_get_umask_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_GET_UMASK, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "get_umask", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.mask;
}

int guestfs_debug_upload (guestfs_h *g,
		const char *filename,
		const char *tmpname,
		int mode)
{
  struct guestfs_debug_upload_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "debug_upload", "filename");
    return -1;
  }
  if (tmpname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "debug_upload", "tmpname");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "debug_upload");
    fprintf (stderr, " \"%s\"", filename);
    fprintf (stderr, " \"%s\"", tmpname);
    fprintf (stderr, " %d", mode);
    fputc ('\n', stderr);
  }

  if (check_state (g, "debug_upload") == -1) return -1;
  guestfs___set_busy (g);

  args.tmpname = (char *) tmpname;
  args.mode = mode;
  serial = guestfs___send (g, GUESTFS_PROC_DEBUG_UPLOAD,
        (xdrproc_t) xdr_guestfs_debug_upload_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  r = guestfs___send_file (g, filename);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }
  if (r == -2) /* daemon cancelled */
    goto read_reply;

 read_reply:
  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "debug_upload", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DEBUG_UPLOAD, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "debug_upload", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_base64_in (guestfs_h *g,
		const char *base64file,
		const char *filename)
{
  struct guestfs_base64_in_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (base64file == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "base64_in", "base64file");
    return -1;
  }
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "base64_in", "filename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "base64_in");
    fprintf (stderr, " \"%s\"", base64file);
    fprintf (stderr, " \"%s\"", filename);
    fputc ('\n', stderr);
  }

  if (check_state (g, "base64_in") == -1) return -1;
  guestfs___set_busy (g);

  args.filename = (char *) filename;
  serial = guestfs___send (g, GUESTFS_PROC_BASE64_IN,
        (xdrproc_t) xdr_guestfs_base64_in_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  r = guestfs___send_file (g, base64file);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }
  if (r == -2) /* daemon cancelled */
    goto read_reply;

 read_reply:
  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "base64_in", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BASE64_IN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "base64_in", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_base64_out (guestfs_h *g,
		const char *filename,
		const char *base64file)
{
  struct guestfs_base64_out_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "base64_out", "filename");
    return -1;
  }
  if (base64file == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "base64_out", "base64file");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "base64_out");
    fprintf (stderr, " \"%s\"", filename);
    fprintf (stderr, " \"%s\"", base64file);
    fputc ('\n', stderr);
  }

  if (check_state (g, "base64_out") == -1) return -1;
  guestfs___set_busy (g);

  args.filename = (char *) filename;
  serial = guestfs___send (g, GUESTFS_PROC_BASE64_OUT,
        (xdrproc_t) xdr_guestfs_base64_out_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "base64_out", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_BASE64_OUT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "base64_out", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  if (guestfs___recv_file (g, base64file) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_checksums_out (guestfs_h *g,
		const char *csumtype,
		const char *directory,
		const char *sumsfile)
{
  struct guestfs_checksums_out_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (csumtype == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "checksums_out", "csumtype");
    return -1;
  }
  if (directory == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "checksums_out", "directory");
    return -1;
  }
  if (sumsfile == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "checksums_out", "sumsfile");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "checksums_out");
    fprintf (stderr, " \"%s\"", csumtype);
    fprintf (stderr, " \"%s\"", directory);
    fprintf (stderr, " \"%s\"", sumsfile);
    fputc ('\n', stderr);
  }

  if (check_state (g, "checksums_out") == -1) return -1;
  guestfs___set_busy (g);

  args.csumtype = (char *) csumtype;
  args.directory = (char *) directory;
  serial = guestfs___send (g, GUESTFS_PROC_CHECKSUMS_OUT,
        (xdrproc_t) xdr_guestfs_checksums_out_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "checksums_out", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_CHECKSUMS_OUT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "checksums_out", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  if (guestfs___recv_file (g, sumsfile) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_fill_pattern (guestfs_h *g,
		const char *pattern,
		int len,
		const char *path)
{
  struct guestfs_fill_pattern_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (pattern == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fill_pattern", "pattern");
    return -1;
  }
  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fill_pattern", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "fill_pattern");
    fprintf (stderr, " \"%s\"", pattern);
    fprintf (stderr, " %d", len);
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "fill_pattern") == -1) return -1;
  guestfs___set_busy (g);

  args.pattern = (char *) pattern;
  args.len = len;
  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_FILL_PATTERN,
        (xdrproc_t) xdr_guestfs_fill_pattern_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "fill_pattern", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FILL_PATTERN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "fill_pattern", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_write (guestfs_h *g,
		const char *path,
		const char *content,
		size_t content_size)
{
  struct guestfs_write_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "write", "path");
    return -1;
  }
  if (content == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "write", "content");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "write");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " \"%s\"", content);
    fputc ('\n', stderr);
  }

  if (check_state (g, "write") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  /* Just catch grossly large sizes. XDR encoding will make this precise. */
  if (content_size >= GUESTFS_MESSAGE_MAX) {
    error (g, "%s: size of input buffer too large", "write");
    guestfs___end_busy (g);
    return -1;
  }
  args.content.content_val = (char *) content;
  args.content.content_len = content_size;
  serial = guestfs___send (g, GUESTFS_PROC_WRITE,
        (xdrproc_t) xdr_guestfs_write_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "write", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_WRITE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "write", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_pwrite (guestfs_h *g,
		const char *path,
		const char *content,
		size_t content_size,
		int64_t offset)
{
  struct guestfs_pwrite_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_pwrite_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pwrite", "path");
    return -1;
  }
  if (content == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pwrite", "content");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pwrite");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " \"%s\"", content);
    fprintf (stderr, " %" PRIi64, offset);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pwrite") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  /* Just catch grossly large sizes. XDR encoding will make this precise. */
  if (content_size >= GUESTFS_MESSAGE_MAX) {
    error (g, "%s: size of input buffer too large", "pwrite");
    guestfs___end_busy (g);
    return -1;
  }
  args.content.content_val = (char *) content;
  args.content.content_len = content_size;
  args.offset = offset;
  serial = guestfs___send (g, GUESTFS_PROC_PWRITE,
        (xdrproc_t) xdr_guestfs_pwrite_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "pwrite", &hdr, &err,
        (xdrproc_t) xdr_guestfs_pwrite_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PWRITE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pwrite", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.nbytes;
}

int guestfs_resize2fs_size (guestfs_h *g,
		const char *device,
		int64_t size)
{
  struct guestfs_resize2fs_size_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "resize2fs_size", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "resize2fs_size");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %" PRIi64, size);
    fputc ('\n', stderr);
  }

  if (check_state (g, "resize2fs_size") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.size = size;
  serial = guestfs___send (g, GUESTFS_PROC_RESIZE2FS_SIZE,
        (xdrproc_t) xdr_guestfs_resize2fs_size_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "resize2fs_size", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_RESIZE2FS_SIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "resize2fs_size", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_pvresize_size (guestfs_h *g,
		const char *device,
		int64_t size)
{
  struct guestfs_pvresize_size_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pvresize_size", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pvresize_size");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %" PRIi64, size);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pvresize_size") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.size = size;
  serial = guestfs___send (g, GUESTFS_PROC_PVRESIZE_SIZE,
        (xdrproc_t) xdr_guestfs_pvresize_size_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "pvresize_size", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PVRESIZE_SIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pvresize_size", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_ntfsresize_size (guestfs_h *g,
		const char *device,
		int64_t size)
{
  struct guestfs_ntfsresize_size_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "ntfsresize_size", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "ntfsresize_size");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %" PRIi64, size);
    fputc ('\n', stderr);
  }

  if (check_state (g, "ntfsresize_size") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.size = size;
  serial = guestfs___send (g, GUESTFS_PROC_NTFSRESIZE_SIZE,
        (xdrproc_t) xdr_guestfs_ntfsresize_size_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "ntfsresize_size", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_NTFSRESIZE_SIZE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "ntfsresize_size", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char **guestfs_available_all_groups (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_available_all_groups_ret ret;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "available_all_groups");
    fputc ('\n', stderr);
  }

  if (check_state (g, "available_all_groups") == -1) return NULL;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_AVAILABLE_ALL_GROUPS, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "available_all_groups", &hdr, &err,
        (xdrproc_t) xdr_guestfs_available_all_groups_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_AVAILABLE_ALL_GROUPS, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "available_all_groups", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* caller will free this, but we need to add a NULL entry */
  ret.groups.groups_val =
    safe_realloc (g, ret.groups.groups_val,
                  sizeof (char *) * (ret.groups.groups_len + 1));
  ret.groups.groups_val[ret.groups.groups_len] = NULL;
  return ret.groups.groups_val;
}

int guestfs_fallocate64 (guestfs_h *g,
		const char *path,
		int64_t len)
{
  struct guestfs_fallocate64_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "fallocate64", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "fallocate64");
    fprintf (stderr, " \"%s\"", path);
    fprintf (stderr, " %" PRIi64, len);
    fputc ('\n', stderr);
  }

  if (check_state (g, "fallocate64") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  args.len = len;
  serial = guestfs___send (g, GUESTFS_PROC_FALLOCATE64,
        (xdrproc_t) xdr_guestfs_fallocate64_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "fallocate64", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FALLOCATE64, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "fallocate64", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

char *guestfs_vfs_label (guestfs_h *g,
		const char *device)
{
  struct guestfs_vfs_label_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_vfs_label_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vfs_label", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vfs_label");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vfs_label") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_VFS_LABEL,
        (xdrproc_t) xdr_guestfs_vfs_label_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "vfs_label", &hdr, &err,
        (xdrproc_t) xdr_guestfs_vfs_label_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VFS_LABEL, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vfs_label", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.label; /* caller will free */
}

char *guestfs_vfs_uuid (guestfs_h *g,
		const char *device)
{
  struct guestfs_vfs_uuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_vfs_uuid_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "vfs_uuid", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "vfs_uuid");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "vfs_uuid") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_VFS_UUID,
        (xdrproc_t) xdr_guestfs_vfs_uuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "vfs_uuid", &hdr, &err,
        (xdrproc_t) xdr_guestfs_vfs_uuid_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_VFS_UUID, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "vfs_uuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.uuid; /* caller will free */
}

int guestfs_lvm_set_filter (guestfs_h *g,
		char *const *devices)
{
  struct guestfs_lvm_set_filter_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (devices == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "lvm_set_filter", "devices");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    size_t i;

    fprintf (stderr, "lvm_set_filter");
    fputc (' ', stderr);
    fputc ('"', stderr);
    for (i = 0; devices[i]; ++i) {
      if (i > 0) fputc (' ', stderr);
      fputs (devices[i], stderr);
    }
    fputc ('"', stderr);
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvm_set_filter") == -1) return -1;
  guestfs___set_busy (g);

  args.devices.devices_val = (char **) devices;
  for (args.devices.devices_len = 0; devices[args.devices.devices_len]; args.devices.devices_len++) ;
  serial = guestfs___send (g, GUESTFS_PROC_LVM_SET_FILTER,
        (xdrproc_t) xdr_guestfs_lvm_set_filter_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lvm_set_filter", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVM_SET_FILTER, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvm_set_filter", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_lvm_clear_filter (guestfs_h *g)
{
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "lvm_clear_filter");
    fputc ('\n', stderr);
  }

  if (check_state (g, "lvm_clear_filter") == -1) return -1;
  guestfs___set_busy (g);

  serial = guestfs___send (g, GUESTFS_PROC_LVM_CLEAR_FILTER, NULL, NULL);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "lvm_clear_filter", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LVM_CLEAR_FILTER, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "lvm_clear_filter", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_luks_open (guestfs_h *g,
		const char *device,
		const char *key,
		const char *mapname)
{
  struct guestfs_luks_open_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_open", "device");
    return -1;
  }
  if (key == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_open", "key");
    return -1;
  }
  if (mapname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_open", "mapname");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "luks_open");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", key);
    fprintf (stderr, " \"%s\"", mapname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "luks_open") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.key = (char *) key;
  args.mapname = (char *) mapname;
  serial = guestfs___send (g, GUESTFS_PROC_LUKS_OPEN,
        (xdrproc_t) xdr_guestfs_luks_open_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "luks_open", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LUKS_OPEN, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "luks_open", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_luks_open_ro (guestfs_h *g,
		const char *device,
		const char *key,
		const char *mapname)
{
  struct guestfs_luks_open_ro_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_open_ro", "device");
    return -1;
  }
  if (key == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_open_ro", "key");
    return -1;
  }
  if (mapname == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_open_ro", "mapname");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "luks_open_ro");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", key);
    fprintf (stderr, " \"%s\"", mapname);
    fputc ('\n', stderr);
  }

  if (check_state (g, "luks_open_ro") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.key = (char *) key;
  args.mapname = (char *) mapname;
  serial = guestfs___send (g, GUESTFS_PROC_LUKS_OPEN_RO,
        (xdrproc_t) xdr_guestfs_luks_open_ro_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "luks_open_ro", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LUKS_OPEN_RO, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "luks_open_ro", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_luks_close (guestfs_h *g,
		const char *device)
{
  struct guestfs_luks_close_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_close", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "luks_close");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "luks_close") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_LUKS_CLOSE,
        (xdrproc_t) xdr_guestfs_luks_close_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "luks_close", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LUKS_CLOSE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "luks_close", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_luks_format (guestfs_h *g,
		const char *device,
		const char *key,
		int keyslot)
{
  struct guestfs_luks_format_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_format", "device");
    return -1;
  }
  if (key == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_format", "key");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "luks_format");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", key);
    fprintf (stderr, " %d", keyslot);
    fputc ('\n', stderr);
  }

  if (check_state (g, "luks_format") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.key = (char *) key;
  args.keyslot = keyslot;
  serial = guestfs___send (g, GUESTFS_PROC_LUKS_FORMAT,
        (xdrproc_t) xdr_guestfs_luks_format_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "luks_format", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LUKS_FORMAT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "luks_format", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_luks_format_cipher (guestfs_h *g,
		const char *device,
		const char *key,
		int keyslot,
		const char *cipher)
{
  struct guestfs_luks_format_cipher_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_format_cipher", "device");
    return -1;
  }
  if (key == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_format_cipher", "key");
    return -1;
  }
  if (cipher == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_format_cipher", "cipher");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "luks_format_cipher");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", key);
    fprintf (stderr, " %d", keyslot);
    fprintf (stderr, " \"%s\"", cipher);
    fputc ('\n', stderr);
  }

  if (check_state (g, "luks_format_cipher") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.key = (char *) key;
  args.keyslot = keyslot;
  args.cipher = (char *) cipher;
  serial = guestfs___send (g, GUESTFS_PROC_LUKS_FORMAT_CIPHER,
        (xdrproc_t) xdr_guestfs_luks_format_cipher_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "luks_format_cipher", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LUKS_FORMAT_CIPHER, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "luks_format_cipher", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_luks_add_key (guestfs_h *g,
		const char *device,
		const char *key,
		const char *newkey,
		int keyslot)
{
  struct guestfs_luks_add_key_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_add_key", "device");
    return -1;
  }
  if (key == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_add_key", "key");
    return -1;
  }
  if (newkey == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_add_key", "newkey");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "luks_add_key");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", key);
    fprintf (stderr, " \"%s\"", newkey);
    fprintf (stderr, " %d", keyslot);
    fputc ('\n', stderr);
  }

  if (check_state (g, "luks_add_key") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.key = (char *) key;
  args.newkey = (char *) newkey;
  args.keyslot = keyslot;
  serial = guestfs___send (g, GUESTFS_PROC_LUKS_ADD_KEY,
        (xdrproc_t) xdr_guestfs_luks_add_key_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "luks_add_key", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LUKS_ADD_KEY, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "luks_add_key", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_luks_kill_slot (guestfs_h *g,
		const char *device,
		const char *key,
		int keyslot)
{
  struct guestfs_luks_kill_slot_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_kill_slot", "device");
    return -1;
  }
  if (key == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "luks_kill_slot", "key");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "luks_kill_slot");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", key);
    fprintf (stderr, " %d", keyslot);
    fputc ('\n', stderr);
  }

  if (check_state (g, "luks_kill_slot") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.key = (char *) key;
  args.keyslot = keyslot;
  serial = guestfs___send (g, GUESTFS_PROC_LUKS_KILL_SLOT,
        (xdrproc_t) xdr_guestfs_luks_kill_slot_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "luks_kill_slot", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_LUKS_KILL_SLOT, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "luks_kill_slot", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_is_lv (guestfs_h *g,
		const char *device)
{
  struct guestfs_is_lv_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_is_lv_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "is_lv", "device");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_lv");
    fprintf (stderr, " \"%s\"", device);
    fputc ('\n', stderr);
  }

  if (check_state (g, "is_lv") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  serial = guestfs___send (g, GUESTFS_PROC_IS_LV,
        (xdrproc_t) xdr_guestfs_is_lv_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "is_lv", &hdr, &err,
        (xdrproc_t) xdr_guestfs_is_lv_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_IS_LV, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "is_lv", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.lvflag;
}

char *guestfs_findfs_uuid (guestfs_h *g,
		const char *uuid)
{
  struct guestfs_findfs_uuid_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_findfs_uuid_ret ret;
  int serial;
  int r;

  if (uuid == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "findfs_uuid", "uuid");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "findfs_uuid");
    fprintf (stderr, " \"%s\"", uuid);
    fputc ('\n', stderr);
  }

  if (check_state (g, "findfs_uuid") == -1) return NULL;
  guestfs___set_busy (g);

  args.uuid = (char *) uuid;
  serial = guestfs___send (g, GUESTFS_PROC_FINDFS_UUID,
        (xdrproc_t) xdr_guestfs_findfs_uuid_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "findfs_uuid", &hdr, &err,
        (xdrproc_t) xdr_guestfs_findfs_uuid_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FINDFS_UUID, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "findfs_uuid", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.device; /* caller will free */
}

char *guestfs_findfs_label (guestfs_h *g,
		const char *label)
{
  struct guestfs_findfs_label_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_findfs_label_ret ret;
  int serial;
  int r;

  if (label == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "findfs_label", "label");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "findfs_label");
    fprintf (stderr, " \"%s\"", label);
    fputc ('\n', stderr);
  }

  if (check_state (g, "findfs_label") == -1) return NULL;
  guestfs___set_busy (g);

  args.label = (char *) label;
  serial = guestfs___send (g, GUESTFS_PROC_FINDFS_LABEL,
        (xdrproc_t) xdr_guestfs_findfs_label_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "findfs_label", &hdr, &err,
        (xdrproc_t) xdr_guestfs_findfs_label_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_FINDFS_LABEL, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "findfs_label", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.device; /* caller will free */
}

int guestfs_is_chardev (guestfs_h *g,
		const char *path)
{
  struct guestfs_is_chardev_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_is_chardev_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "is_chardev", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_chardev");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "is_chardev") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_IS_CHARDEV,
        (xdrproc_t) xdr_guestfs_is_chardev_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "is_chardev", &hdr, &err,
        (xdrproc_t) xdr_guestfs_is_chardev_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_IS_CHARDEV, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "is_chardev", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.flag;
}

int guestfs_is_blockdev (guestfs_h *g,
		const char *path)
{
  struct guestfs_is_blockdev_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_is_blockdev_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "is_blockdev", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_blockdev");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "is_blockdev") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_IS_BLOCKDEV,
        (xdrproc_t) xdr_guestfs_is_blockdev_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "is_blockdev", &hdr, &err,
        (xdrproc_t) xdr_guestfs_is_blockdev_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_IS_BLOCKDEV, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "is_blockdev", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.flag;
}

int guestfs_is_fifo (guestfs_h *g,
		const char *path)
{
  struct guestfs_is_fifo_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_is_fifo_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "is_fifo", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_fifo");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "is_fifo") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_IS_FIFO,
        (xdrproc_t) xdr_guestfs_is_fifo_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "is_fifo", &hdr, &err,
        (xdrproc_t) xdr_guestfs_is_fifo_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_IS_FIFO, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "is_fifo", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.flag;
}

int guestfs_is_symlink (guestfs_h *g,
		const char *path)
{
  struct guestfs_is_symlink_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_is_symlink_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "is_symlink", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_symlink");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "is_symlink") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_IS_SYMLINK,
        (xdrproc_t) xdr_guestfs_is_symlink_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "is_symlink", &hdr, &err,
        (xdrproc_t) xdr_guestfs_is_symlink_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_IS_SYMLINK, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "is_symlink", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.flag;
}

int guestfs_is_socket (guestfs_h *g,
		const char *path)
{
  struct guestfs_is_socket_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_is_socket_ret ret;
  int serial;
  int r;

  if (path == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "is_socket", "path");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "is_socket");
    fprintf (stderr, " \"%s\"", path);
    fputc ('\n', stderr);
  }

  if (check_state (g, "is_socket") == -1) return -1;
  guestfs___set_busy (g);

  args.path = (char *) path;
  serial = guestfs___send (g, GUESTFS_PROC_IS_SOCKET,
        (xdrproc_t) xdr_guestfs_is_socket_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "is_socket", &hdr, &err,
        (xdrproc_t) xdr_guestfs_is_socket_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_IS_SOCKET, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "is_socket", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.flag;
}

char *guestfs_part_to_dev (guestfs_h *g,
		const char *partition)
{
  struct guestfs_part_to_dev_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_part_to_dev_ret ret;
  int serial;
  int r;

  if (partition == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "part_to_dev", "partition");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "part_to_dev");
    fprintf (stderr, " \"%s\"", partition);
    fputc ('\n', stderr);
  }

  if (check_state (g, "part_to_dev") == -1) return NULL;
  guestfs___set_busy (g);

  args.partition = (char *) partition;
  serial = guestfs___send (g, GUESTFS_PROC_PART_TO_DEV,
        (xdrproc_t) xdr_guestfs_part_to_dev_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "part_to_dev", &hdr, &err,
        (xdrproc_t) xdr_guestfs_part_to_dev_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PART_TO_DEV, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "part_to_dev", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  return ret.device; /* caller will free */
}

int guestfs_upload_offset (guestfs_h *g,
		const char *filename,
		const char *remotefilename,
		int64_t offset)
{
  struct guestfs_upload_offset_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "upload_offset", "filename");
    return -1;
  }
  if (remotefilename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "upload_offset", "remotefilename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "upload_offset");
    fprintf (stderr, " \"%s\"", filename);
    fprintf (stderr, " \"%s\"", remotefilename);
    fprintf (stderr, " %" PRIi64, offset);
    fputc ('\n', stderr);
  }

  if (check_state (g, "upload_offset") == -1) return -1;
  guestfs___set_busy (g);

  args.remotefilename = (char *) remotefilename;
  args.offset = offset;
  serial = guestfs___send (g, GUESTFS_PROC_UPLOAD_OFFSET,
        (xdrproc_t) xdr_guestfs_upload_offset_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  r = guestfs___send_file (g, filename);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }
  if (r == -2) /* daemon cancelled */
    goto read_reply;

 read_reply:
  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "upload_offset", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_UPLOAD_OFFSET, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "upload_offset", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_download_offset (guestfs_h *g,
		const char *remotefilename,
		const char *filename,
		int64_t offset,
		int64_t size)
{
  struct guestfs_download_offset_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  int serial;
  int r;

  if (remotefilename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "download_offset", "remotefilename");
    return -1;
  }
  if (filename == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "download_offset", "filename");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "download_offset");
    fprintf (stderr, " \"%s\"", remotefilename);
    fprintf (stderr, " \"%s\"", filename);
    fprintf (stderr, " %" PRIi64, offset);
    fprintf (stderr, " %" PRIi64, size);
    fputc ('\n', stderr);
  }

  if (check_state (g, "download_offset") == -1) return -1;
  guestfs___set_busy (g);

  args.remotefilename = (char *) remotefilename;
  args.offset = offset;
  args.size = size;
  serial = guestfs___send (g, GUESTFS_PROC_DOWNLOAD_OFFSET,
        (xdrproc_t) xdr_guestfs_download_offset_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);

  r = guestfs___recv (g, "download_offset", &hdr, &err,
        NULL, NULL);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_DOWNLOAD_OFFSET, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "download_offset", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  if (guestfs___recv_file (g, filename) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return 0;
}

int guestfs_pwrite_device (guestfs_h *g,
		const char *device,
		const char *content,
		size_t content_size,
		int64_t offset)
{
  struct guestfs_pwrite_device_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_pwrite_device_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pwrite_device", "device");
    return -1;
  }
  if (content == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pwrite_device", "content");
    return -1;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pwrite_device");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " \"%s\"", content);
    fprintf (stderr, " %" PRIi64, offset);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pwrite_device") == -1) return -1;
  guestfs___set_busy (g);

  args.device = (char *) device;
  /* Just catch grossly large sizes. XDR encoding will make this precise. */
  if (content_size >= GUESTFS_MESSAGE_MAX) {
    error (g, "%s: size of input buffer too large", "pwrite_device");
    guestfs___end_busy (g);
    return -1;
  }
  args.content.content_val = (char *) content;
  args.content.content_len = content_size;
  args.offset = offset;
  serial = guestfs___send (g, GUESTFS_PROC_PWRITE_DEVICE,
        (xdrproc_t) xdr_guestfs_pwrite_device_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "pwrite_device", &hdr, &err,
        (xdrproc_t) xdr_guestfs_pwrite_device_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PWRITE_DEVICE, serial) == -1) {
    guestfs___end_busy (g);
    return -1;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pwrite_device", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return -1;
  }

  guestfs___end_busy (g);
  return ret.nbytes;
}

char *guestfs_pread_device (guestfs_h *g,
		const char *device,
		int count,
		int64_t offset,
		size_t *size_r)
{
  struct guestfs_pread_device_args args;
  guestfs_message_header hdr;
  guestfs_message_error err;
  struct guestfs_pread_device_ret ret;
  int serial;
  int r;

  if (device == NULL) {
    error (g, "%s: %s: parameter cannot be NULL",
           "pread_device", "device");
    return NULL;
  }

  if (guestfs__get_trace (g)) {
    fprintf (stderr, "pread_device");
    fprintf (stderr, " \"%s\"", device);
    fprintf (stderr, " %d", count);
    fprintf (stderr, " %" PRIi64, offset);
    fputc ('\n', stderr);
  }

  if (check_state (g, "pread_device") == -1) return NULL;
  guestfs___set_busy (g);

  args.device = (char *) device;
  args.count = count;
  args.offset = offset;
  serial = guestfs___send (g, GUESTFS_PROC_PREAD_DEVICE,
        (xdrproc_t) xdr_guestfs_pread_device_args, (char *) &args);
  if (serial == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  memset (&hdr, 0, sizeof hdr);
  memset (&err, 0, sizeof err);
  memset (&ret, 0, sizeof ret);

  r = guestfs___recv (g, "pread_device", &hdr, &err,
        (xdrproc_t) xdr_guestfs_pread_device_ret, (char *) &ret);
  if (r == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (check_reply_header (g, &hdr, GUESTFS_PROC_PREAD_DEVICE, serial) == -1) {
    guestfs___end_busy (g);
    return NULL;
  }

  if (hdr.status == GUESTFS_STATUS_ERROR) {
    error (g, "%s: %s", "pread_device", err.error_message);
    free (err.error_message);
    guestfs___end_busy (g);
    return NULL;
  }

  guestfs___end_busy (g);
  /* RBufferOut is tricky: If the buffer is zero-length, then
   * _val might be NULL here.  To make the API saner for
   * callers, we turn this case into a unique pointer (using
   * malloc(1)).
   */
  if (ret.content.content_len > 0) {
    *size_r = ret.content.content_len;
    return ret.content.content_val; /* caller will free */
  } else {
    free (ret.content.content_val);
    char *p = safe_malloc (g, 1);
    *size_r = ret.content.content_len;
    return p;
  }
}

/* Structure-freeing functions.  These rely on the fact that the
 * structure format is identical to the XDR format.  See note in
 * generator.ml.
 */

void
guestfs_free_int_bool (struct guestfs_int_bool *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_int_bool, (char *) x);
  free (x);
}

void
guestfs_free_int_bool_list (struct guestfs_int_bool_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_int_bool_list, (char *) x);
  free (x);
}

void
guestfs_free_lvm_pv (struct guestfs_lvm_pv *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_lvm_pv, (char *) x);
  free (x);
}

void
guestfs_free_lvm_pv_list (struct guestfs_lvm_pv_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_lvm_pv_list, (char *) x);
  free (x);
}

void
guestfs_free_lvm_vg (struct guestfs_lvm_vg *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_lvm_vg, (char *) x);
  free (x);
}

void
guestfs_free_lvm_vg_list (struct guestfs_lvm_vg_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_lvm_vg_list, (char *) x);
  free (x);
}

void
guestfs_free_lvm_lv (struct guestfs_lvm_lv *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_lvm_lv, (char *) x);
  free (x);
}

void
guestfs_free_lvm_lv_list (struct guestfs_lvm_lv_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_lvm_lv_list, (char *) x);
  free (x);
}

void
guestfs_free_stat (struct guestfs_stat *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_stat, (char *) x);
  free (x);
}

void
guestfs_free_stat_list (struct guestfs_stat_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_stat_list, (char *) x);
  free (x);
}

void
guestfs_free_statvfs (struct guestfs_statvfs *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_statvfs, (char *) x);
  free (x);
}

void
guestfs_free_statvfs_list (struct guestfs_statvfs_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_statvfs_list, (char *) x);
  free (x);
}

void
guestfs_free_dirent (struct guestfs_dirent *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_dirent, (char *) x);
  free (x);
}

void
guestfs_free_dirent_list (struct guestfs_dirent_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_dirent_list, (char *) x);
  free (x);
}

void
guestfs_free_version (struct guestfs_version *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_version, (char *) x);
  free (x);
}

void
guestfs_free_version_list (struct guestfs_version_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_version_list, (char *) x);
  free (x);
}

void
guestfs_free_xattr (struct guestfs_xattr *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_xattr, (char *) x);
  free (x);
}

void
guestfs_free_xattr_list (struct guestfs_xattr_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_xattr_list, (char *) x);
  free (x);
}

void
guestfs_free_inotify_event (struct guestfs_inotify_event *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_inotify_event, (char *) x);
  free (x);
}

void
guestfs_free_inotify_event_list (struct guestfs_inotify_event_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_inotify_event_list, (char *) x);
  free (x);
}

void
guestfs_free_partition (struct guestfs_partition *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_partition, (char *) x);
  free (x);
}

void
guestfs_free_partition_list (struct guestfs_partition_list *x)
{
  xdr_free ((xdrproc_t) xdr_guestfs_int_partition_list, (char *) x);
  free (x);
}

